﻿using FlatRedBall;
using Maze.Helpers;
using Maze.Models;

namespace Maze.Extensions
{
    public static class PositionedObjectExtensions
    {
        public static bool IsMoving(this PositionedObject obj)
        {
            bool xVelocityIsZero = obj.Velocity.X.AlmostEquals(0, 5f);
            bool yVelocityIsZero = obj.Velocity.Y.AlmostEquals(0, 5f);
            return !xVelocityIsZero || !yVelocityIsZero;
        }

        public static DirectionHelper.Direction GetMovementDirection(this PositionedObject obj)
        {
            if (!obj.IsMoving())
            {
                return DirectionHelper.Direction.None;
            }
            if (PositionedObjectHelper.GoingNorth(obj))
            {
                if (PositionedObjectHelper.GoingNorthEast(obj))
                {
                    return DirectionHelper.Direction.NorthEast;
                }
                return PositionedObjectHelper.GoingNorthWest(obj) ? DirectionHelper.Direction.NorthWest : DirectionHelper.Direction.North;
            }
            if (PositionedObjectHelper.GoingSouth(obj))
            {
                if (PositionedObjectHelper.GoingSouthEast(obj))
                {
                    return DirectionHelper.Direction.SouthEast;
                }
                return PositionedObjectHelper.GoingSouthWest(obj) ? DirectionHelper.Direction.SouthWest : DirectionHelper.Direction.South;
            }
            if (PositionedObjectHelper.GoingEast(obj))
            {
                return DirectionHelper.Direction.East;
            }
            if (PositionedObjectHelper.GoingWest(obj))
            {
                return DirectionHelper.Direction.West;
            }
            return DirectionHelper.Direction.None;
        }

        public static DirectionHelper.Direction GetRealMovementDirection(this PositionedObject obj, RotatingCamera camera)
        {
            return !obj.IsMoving()
                ? DirectionHelper.Direction.None
                : PositionedObjectHelper.TranslateMovementDirectionToRealDirection(camera.CameraAngle, obj.GetMovementDirection());
        }
    }
}