﻿using System;
using System.Collections.Generic;
using System.Linq;
using FlatRedBall;
using Maze.Models;

namespace Maze.Helpers
{
    public static class CameraHelper
    {
        private static readonly Dictionary<float, DirectionHelper.CameraAngle> CardinalPoints;

        static CameraHelper()
        {
            CardinalPoints = CreateCardinalDirectionDictionary();
        }

        public static RotatingCamera CreateRotatingCamera(string contentManager)
        {
            var camera = new RotatingCamera(contentManager);
            camera.DrawsWorld = false;
            SpriteManager.Cameras.Add(camera);
            return camera;
        }

        public static void DestroyCamera(Camera camera)
        {
            SpriteManager.Cameras.Remove(camera);

            if (camera is RotatingCamera)
            {
                (camera as RotatingCamera).Destroy();
            }

            camera = null;
        }

        public static DirectionHelper.CameraAngle GetCameraAngle(RotatingCamera camera)
        {
            var heading = camera.Rotation;
            return FindClosest(CardinalPoints, heading);
        }

        private static Dictionary<float, DirectionHelper.CameraAngle> CreateCardinalDirectionDictionary()
        {
            var cardinalPoints = new Dictionary<float, DirectionHelper.CameraAngle>();
        
            var cardinalDirectionPoints = Enum.GetNames(typeof(DirectionHelper.CameraAngle)).Length;
        
            var degreesBetweenEachPoint = 360 / cardinalDirectionPoints;
        
            for (int index = 0; index < cardinalDirectionPoints; index++)
            {
                cardinalPoints.Add(index * degreesBetweenEachPoint, (DirectionHelper.CameraAngle)index);
            }
        
            return cardinalPoints;
        }

        private static DirectionHelper.CameraAngle FindClosest(IEnumerable<KeyValuePair<float, DirectionHelper.CameraAngle>> cardinalPoints, float number)
        {
            const int majorAngleWidth = 5;

            var closestItem = cardinalPoints.Aggregate((x, y) => Math.Abs(x.Key - number) < Math.Abs(y.Key - number) ? x : y);

            switch (closestItem.Value)
            {
                case DirectionHelper.CameraAngle.North:
                case DirectionHelper.CameraAngle.East:
                case DirectionHelper.CameraAngle.South:
                case DirectionHelper.CameraAngle.West:
                    var minimumAngle = closestItem.Key - majorAngleWidth;
                    var maximumAngle = closestItem.Key + majorAngleWidth;

                    if (number >= minimumAngle && number <= maximumAngle)
                    {
                        return closestItem.Value;
                    }

                    if (number < minimumAngle)
                    {
                        var newItem = (int) closestItem.Value - 1;
                        return GetCameraAngle(newItem);
                    }

                    if (number > maximumAngle)
                    {
                        var newItem = (int) closestItem.Value + 1;
                        return GetCameraAngle(newItem);
                    }
                    break;
                case DirectionHelper.CameraAngle.NorthWest:
                    // Special case for NorthWest, since both 0 and 360 degreees is North.
                    if (number > (360 - majorAngleWidth))
                    {
                        return DirectionHelper.CameraAngle.North;
                    }
                    break;
            }

            return closestItem.Value;
        }

        private static DirectionHelper.CameraAngle GetCameraAngle(int item)
        {
                if (item < 0)
                {
                    item = Enum.GetNames(typeof(DirectionHelper.CameraAngle)).Length;
                }
                if (item > Enum.GetNames(typeof(DirectionHelper.CameraAngle)).Length)
                {
                    item = 0;
                }
                return (DirectionHelper.CameraAngle) item;
            }
        }
}
