﻿using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.ManagedSpriteGroups;
using FlatRedBall.Math.Geometry;
using Maze.Models;
using Maze.Models.Items;
using Microsoft.Xna.Framework;

namespace Maze.Helpers
{
    public static class LevelHelper
    {
        private const string LevelDirectory = @"Content\Levels\";

        public static Scene LoadScene(string path, string contentManager, Layer layer = null)
        {
            path = LevelDirectory + path;

            var scene = FlatRedBallServices.Load<Scene>(path, contentManager);
            scene.AddToManagers(layer);
            return scene;
        }

        public static ShapeCollection LoadShapeCollection(string path, string contentManager, Layer layer = null)
        {
            path = LevelDirectory + path;

            var shapeCollection = FlatRedBallServices.Load<ShapeCollection>(path, contentManager);
            shapeCollection.AddToManagers(layer);
            return shapeCollection;
        }

        public static void MoveSpriteGrid(SpriteGrid grid, Vector2 coordinate)
        {
            MoveSpriteGrid(grid, coordinate.X, coordinate.Y);
        }

        public static void MoveSpriteGrid(SpriteGrid grid, float centerX, float centerY)
        {
            var gridWidth = GetSpriteGridWidth(grid);
            var gridHeight = GetSpriteGridHeight(grid);

            var shiftX = centerX - gridWidth / 2;
            var shiftY = centerY + gridHeight / 2;

            grid.Shift(shiftX, shiftY, 0);

            grid.XLeftBound += shiftX;
            grid.XRightBound += shiftX;
            grid.YTopBound += shiftY;
            grid.YBottomBound += shiftY;
            grid.FillToBounds();
        }

        public static void MoveShapeCollection(ShapeCollection shapes, SpriteGrid grid, Vector2 coordinates)
        {
            MoveShapeCollection(shapes, grid, coordinates.X, coordinates.Y);
        }

        public static void MoveShapeCollection(ShapeCollection shapes, SpriteGrid grid, float centerX, float centerY)
        {
            shapes.Shift(new Vector3(centerX - GetSpriteGridWidth(grid) / 2, centerY + GetSpriteGridHeight(grid) / 2, 0));
        }

        public static void MoveSprites(SpriteList sprites, SpriteGrid grid, Vector2 coordinates)
        {
            MoveSprites(sprites, grid, coordinates.X, coordinates.Y);
        }

        public static void MoveSprites(SpriteList sprites, SpriteGrid grid, float centerX, float centerY)
        {
            var shiftX = centerX - GetSpriteGridWidth(grid) / 2;
            var shiftY = centerY + GetSpriteGridHeight(grid) / 2;

            sprites.Shift(shiftX, shiftY, 0);
        }

        public static void MoveFinishPoints(FinishPoints finishPoints, SpriteGrid grid, Vector2 coordinates)
        {
            MoveFinishPoints(finishPoints, grid, coordinates.X, coordinates.Y);
        }

        public static void MoveFinishPoints(FinishPoints finishPoints, SpriteGrid grid, float centerX, float centerY)
        {
            var shiftX = centerX - GetSpriteGridWidth(grid) / 2;
            var shiftY = centerY + GetSpriteGridHeight(grid) / 2;

            finishPoints.Shift(shiftX, shiftY, 0);
        }

        public static void MoveItems(ItemList items, SpriteGrid grid, Vector2 coordinates)
        {
            MoveItems(items, grid, coordinates.X, coordinates.Y);
        }

        public static void MoveItems(ItemList items, SpriteGrid grid, float centerX, float centerY)
        {
            var shiftX = centerX - GetSpriteGridWidth(grid) / 2;
            var shiftY = centerY + GetSpriteGridHeight(grid) / 2;

            items.Shift(shiftX, shiftY, 0);
        }

        private static float GetSpriteGridHeight(SpriteGrid grid)
        {
            var gridHeight = grid.FurthestTopY - grid.FurthestBottomY;
            return gridHeight;
        }

        private static float GetSpriteGridWidth(SpriteGrid grid)
        {
            var gridWidth = grid.FurthestRightX - grid.FurthestLeftX;
            return gridWidth;
        }
    }
}
