﻿using System;
using FlatRedBall;

namespace Maze.Helpers
{
    public static class PositionedObjectHelper
    {
        public static DirectionHelper.Direction TranslateMovementDirectionToRealDirection(DirectionHelper.CameraAngle cameraAngle, DirectionHelper.Direction objectDirection)
        {
            const int invalidDirectionsCount = 1;
            var validDirectionsCount = Enum.GetNames(typeof(DirectionHelper.Direction)).Length - invalidDirectionsCount;

            if (cameraAngle == DirectionHelper.CameraAngle.North || objectDirection == DirectionHelper.Direction.None)
            {
                return objectDirection;
            }

            var shift = (int)objectDirection + (int)cameraAngle;

            if (shift == validDirectionsCount)
            {
                shift = 0;
            }

            if (shift > validDirectionsCount)
            {
                shift = shift - validDirectionsCount;
            }

            return (DirectionHelper.Direction)shift;
        }

        public static bool GoingNorth(PositionedObject obj)
        {
            return obj.Velocity.Y > 0;
        }

        public static bool GoingSouth(PositionedObject obj)
        {
            return obj.Velocity.Y < 0;
        }

        public static bool GoingEast(PositionedObject obj)
        {
            return obj.Velocity.X < 0;
        }

        public static bool GoingWest(PositionedObject obj)
        {
            return obj.Velocity.X > 0;
        }

        public static bool GoingNorthEast(PositionedObject obj)
        {
            return GoingNorth(obj) && GoingEast(obj);
        }

        public static bool GoingNorthWest(PositionedObject obj)
        {
            return GoingNorth(obj) && GoingWest(obj);
        }

        public static bool GoingSouthEast(PositionedObject obj)
        {
            return GoingSouth(obj) && GoingEast(obj);
        }

        public static bool GoingSouthWest(PositionedObject obj)
        {
            return GoingSouth(obj) && GoingWest(obj);
        }
    }
}
