﻿using System;
using Maze.Input.Kinect;
using Maze.Input.Move;
using Maze.Input.Wii;

namespace Maze.Input
{
    class InputDetecter
    {
        private static InputDetecter _instance;
        
        private readonly WiiDetecter _wiiDetecter;
        private readonly KinectDetecter _kinectDetecter;
        private readonly MoveDetecter _moveDetecter;


        protected InputDetecter()
        {
            _wiiDetecter = WiiDetecter.GetInstance();
            _kinectDetecter = KinectDetecter.GetInstance();
            _moveDetecter = MoveDetecter.GetInstance();
        }

        public static InputDetecter GetInstance()
        {
            return _instance ?? (_instance = new InputDetecter());
        }

        public Input DetectInputs()
        {
            var detectedInput = Input.None;

            if (IsKeyboardPresent())
            {
                detectedInput = detectedInput | Input.Keyboard;
            }
            if (IsWiiPresent())
            {
                detectedInput = detectedInput | Input.Wii;
            }
            if (IsKinectPresent())
            {
                detectedInput = detectedInput | Input.Kinect;
            }
            if (IsMovePresent())
            {
                detectedInput = detectedInput | Input.Move;
            }
            Console.WriteLine("Detected the following controllers: " + detectedInput);

            return detectedInput;
        }

        private bool IsKeyboardPresent()
        {
            return true;
        }

        private bool IsWiiPresent()
        {
            _wiiDetecter.LoadWiimotes();
            return _wiiDetecter.WiimoteCount > 0;
        }

        private bool IsKinectPresent()
        {
            return _kinectDetecter.KinectCount > 0;
        }

        private bool IsMovePresent()
        {
            if (!_moveDetecter.Connected)
            {
                _moveDetecter.Connect();
            }
            return _moveDetecter.Moves.Count > 0;
        }

        public void Destroy()
        {
            _wiiDetecter.Destroy();
            _kinectDetecter.Destroy();
            _moveDetecter.Destroy();
        }
    }
}
