﻿using System;
using System.Linq;
using Maze.Input.Events;
using Maze.Input.Kinect.Models;
using Microsoft.Research.Kinect.Nui;

namespace Maze.Input.Kinect
{
    public class KinectDetecter
    {
        private static KinectDetecter _instance;

        private readonly InputDetectedEvent _inputDetectedEvent;

        protected KinectDetecter()
        {
            Kinects = new KinectCollection();
            _inputDetectedEvent = new InputDetectedEvent();

            DetectKinects();
            SubscribeToEvents();
        }

        public static KinectDetecter GetInstance()
        {
            return _instance ?? (_instance = new KinectDetecter());
        }

        public int KinectCount
        {
            get
            {
                return Kinects.Count;
            }
        }

        public KinectCollection Kinects
        {
            get;
            private set;
        }

        private void DetectKinects()
        {
            if (Runtime.Kinects.Count > 0)
            {
                foreach (var kinectRuntime in Runtime.Kinects)
                {
                    Kinects.Add(new Models.Kinect(kinectRuntime));
                }
            }
        }

        private void SubscribeToEvents()
        {
            Runtime.Kinects.StatusChanged += KinectStatusChanged;
        }

        private void UnsubscribeFromEvents()
        {
            Runtime.Kinects.StatusChanged -= KinectStatusChanged;
        }

        private void KinectStatusChanged(object sender, StatusChangedEventArgs e)
        {
            switch (e.Status)
            {
                case KinectStatus.Connected:
                    if (!Kinects.Any(k => k.Runtime == e.KinectRuntime))
                    {
                        Kinects.Add(new Models.Kinect(e.KinectRuntime));
                        _inputDetectedEvent.Publish(null);
                    }
                    break;
                case KinectStatus.Disconnected:
                    var kinect = Kinects.Where(k => k.Runtime == e.KinectRuntime).FirstOrDefault();
                    if (kinect != null)
                    {
                        kinect.Destroy();
                        Kinects.Remove(kinect);
                        _inputDetectedEvent.Publish(null);
                    }
                    else
                    {
                        throw new Exception("The disconnected Kinect did not exist in the internal KinectCollection");
                    }
                    break;
            }
        }

        public void Destroy()
        {
            UnsubscribeFromEvents();

            foreach (var kinect in Kinects)
            {
                kinect.Destroy();
            }

            _instance = null;
        }
    }
}
