﻿using System;
using System.Net;
using System.Net.Sockets;
using PSMoveSharp;

namespace Maze.Input.Move
{
    public class MoveClientExtension : PSMoveClient
    {
        public void ConnectWithTimeout(string serverAddress, int serverPort, TimeSpan timout)
        {
            if (timout.TotalMilliseconds < 1)
            {
                throw new Exception("Invalid timeout value.");
            }

            _tcpClient = new TcpClient();
            DoTcpConnect(serverAddress, serverPort, timout);

            _udpClient = new UdpClient(0);
            Console.WriteLine("Initial recieve buffer size: {0}", _udpClient.Client.ReceiveBufferSize);
            _udpClient.Client.ReceiveBufferSize = 655360; // 640 KB
            Console.WriteLine("Expanded recieve buffer size: {0}", _udpClient.Client.ReceiveBufferSize);
            uint udpport = (uint)((IPEndPoint)_udpClient.Client.LocalEndPoint).Port;
            SendRequestPacket(ClientRequest.PSMoveClientRequestInit, udpport);
        }

        private void DoTcpConnect(string serverAddress, int serverPort, TimeSpan timout)
        {
            var asyncResult = _tcpClient.BeginConnect(serverAddress, serverPort, null, null);
            var timeoutValue = timout;
            try
            {
                if (!asyncResult.AsyncWaitHandle.WaitOne(timeoutValue, false))
                {
                    _tcpClient.Close();
                    throw new TimeoutException();
                }
                _tcpClient.EndConnect(asyncResult);
            }
            finally
            {
                asyncResult.AsyncWaitHandle.Close();
            }
        }
    }
}
