﻿using System;
using Maze.Converters;
using Maze.Input.Wii.Algorithms;
using WiimoteLib;


namespace Maze.Input.Wii.Converters
{
    public class ToDegreesConverter : BaseConverter<Wiimote, float>
    {
        private readonly MadgwickAHRS _ahrs;

        public ToDegreesConverter()
        {
            // Wiimote operates at 100Hz
            _ahrs = new MadgwickAHRS(1f / 100f, 0.5f);
        }

        public override Wiimote ConvertFrom(float convert)
        {
            throw new NotImplementedException();
        }

        public override float ConvertTo(Wiimote convert)
        {
            var wiimoteState = convert.WiimoteState;

            var wiiYaw = wiimoteState.MotionPlusState.GyroState.Values.Yaw;
            var wiiPitch = wiimoteState.MotionPlusState.GyroState.Values.Pitch;
            var wiiRoll = wiimoteState.MotionPlusState.GyroState.Values.Roll;
            var acceleration = wiimoteState.AccelState.Values;

            _ahrs.Update(Deg2Rad(wiiPitch), Deg2Rad(wiiRoll), Deg2Rad(wiiYaw), acceleration.X, acceleration.Y, acceleration.Z);

            var quat = _ahrs.Quaternion;

            var pitch = Math.Atan2(2 * (quat[0] * quat[1] + quat[2] * quat[3]), 1 - 2 * (quat[1] * quat[1] + quat[2] * quat[2]));
//            var roll = Math.Asin(2 * (quat[0] * quat[2] - quat[3] * quat[1]));
//            var yaw = Math.Atan2(2 * (quat[0] * quat[3] + quat[1] * quat[2]), 1 - 2 * (quat[2] * quat[2] + quat[3] * quat[3]));

            var angle = pitch * (180 / Math.PI);

            return (float) angle;
        }

        /// <summary>
        /// Converts degrees to radians.
        /// </summary>
        /// <param name="degrees">
        /// Angular quantity in degrees.
        /// </param>
        /// <returns>
        /// Angular quantity in radians.
        /// </returns>
        private static float Deg2Rad(float degrees)
        {
            return (float)(Math.PI / 180) * degrees;
        }
    }
}
