﻿using System;
using System.Collections.Generic;
using Maze.Input.Wii.Models.Extensions;
using WiimoteLib;

namespace Maze.Input.Wii.Extensions
{
    public static class WiimoteExtensions
    {
        private static readonly Dictionary<Wiimote, MotionPlus> MotionPlus;
        private static readonly Dictionary<Wiimote, BalanceBoard> BalanceBoards;
        private static readonly Dictionary<WiiButton, DateTime> Quarantine;
        private static readonly TimeSpan QuarantineAmount;

        static WiimoteExtensions()
        {
            Quarantine = new Dictionary<WiiButton, DateTime>();
            QuarantineAmount = new TimeSpan(0, 0, 0, 0, 200);

            MotionPlus = new Dictionary<Wiimote, MotionPlus>();
            BalanceBoards = new Dictionary<Wiimote, BalanceBoard>();
        }

        public static bool HasExtension(this Wiimote wiimote)
        {
            return wiimote.WiimoteState.Extension;
        }

        public static ExtensionType GetExtensionType(this Wiimote wiimote)
        {
            return wiimote.WiimoteState.ExtensionType;
        }

        public static BalanceBoard GetBalanceBoard(this Wiimote wiimote)
        {
            if (!BalanceBoards.ContainsKey(wiimote))
            {
                BalanceBoards.Add(wiimote, new BalanceBoard(wiimote));
            }
            return BalanceBoards[wiimote];
        }

        public static bool HasMotionPlus(this Wiimote wiimote)
        {
            return wiimote.HasExtension() && wiimote.GetExtensionType() == ExtensionType.MotionPlus;
        }

        public static MotionPlus GetMotionPlus(this Wiimote wiimote)
        {
            return wiimote.HasMotionPlus() ? wiimote.RetrieveMotionPlus() : null;
        }

        public static MotionPlus RetrieveMotionPlus(this Wiimote wiimote)
        {
            return MotionPlus[wiimote];
        }

        public static void SetMotionPlus(this Wiimote wiimote, MotionPlus motionPlus)
        {
            MotionPlus[wiimote] = motionPlus;
        }

        public static bool KeyPushed(this Wiimote wiimote, WiiButton button)
        {
            var buttonState = wiimote.WiimoteState.ButtonState;

            bool isButtonPushed;
            switch (button)
            {
                case WiiButton.A:
                    isButtonPushed = CheckIsPushed(button, buttonState.A);
                    break;
                case WiiButton.B:
                    isButtonPushed = CheckIsPushed(button, buttonState.B);
                    break;
                case WiiButton.Minus:
                    isButtonPushed = CheckIsPushed(button, buttonState.Minus);
                    break;
                case WiiButton.Home:
                    isButtonPushed = CheckIsPushed(button, buttonState.Home);
                    break;
                case WiiButton.Plus:
                    isButtonPushed = CheckIsPushed(button, buttonState.Plus);
                    break;
                case WiiButton.One:
                    isButtonPushed = CheckIsPushed(button, buttonState.One);
                    break;
                case WiiButton.Two:
                    isButtonPushed = CheckIsPushed(button, buttonState.Two);
                    break;
                case WiiButton.Up:
                    isButtonPushed = CheckIsPushed(button, buttonState.Up);
                    break;
                case WiiButton.Down:
                    isButtonPushed = CheckIsPushed(button, buttonState.Down);
                    break;
                case WiiButton.Left:
                    isButtonPushed = CheckIsPushed(button, buttonState.Left);
                    break;
                case WiiButton.Right:
                    isButtonPushed = CheckIsPushed(button, buttonState.Right);
                    break;
                default:
                    throw new Exception();
            }
            return isButtonPushed;
        }

        public static bool KeyDown(this Wiimote wiimote, WiiButton button)
        {
            var buttonState = wiimote.WiimoteState.ButtonState;

            bool isButtonDown;
            switch (button)
            {
                case WiiButton.A:
                    isButtonDown = buttonState.A;
                    break;
                case WiiButton.B:
                    isButtonDown = buttonState.B;
                    break;
                case WiiButton.Minus:
                    isButtonDown = buttonState.Minus;
                    break;
                case WiiButton.Home:
                    isButtonDown = buttonState.Home;
                    break;
                case WiiButton.Plus:
                    isButtonDown = buttonState.Plus;
                    break;
                case WiiButton.One:
                    isButtonDown = buttonState.One;
                    break;
                case WiiButton.Two:
                    isButtonDown = buttonState.Two;
                    break;
                case WiiButton.Up:
                    isButtonDown = buttonState.Up;
                    break;
                case WiiButton.Down:
                    isButtonDown = buttonState.Down;
                    break;
                case WiiButton.Left:
                    isButtonDown = buttonState.Left;
                    break;
                case WiiButton.Right:
                    isButtonDown = buttonState.Right;
                    break;
                default:
                    throw new Exception();
            }

            return isButtonDown;
        }

        private static bool CheckIsPushed(WiiButton key, bool isButtonPushed)
        {
            if (isButtonPushed)
            {
                var isQuarantined =  Quarantine.ContainsKey(key);
                if (isQuarantined)
                {
                    var shouldRemoveFromQuarantine = DateTime.Now - Quarantine[key] >= QuarantineAmount;
                    if (shouldRemoveFromQuarantine)
                    {
                        Quarantine.Remove(key);
                        isQuarantined = false;
                    }
                }

                if (!isQuarantined)
                {
                    Quarantine.Add(key, DateTime.Now);
                    return true;
                }
            }
            return false;
        }
    }
}
