﻿using System.Timers;

namespace Maze.Input.Wii.Models
{
    class CalibrationTimer
    {
        public event ElapsedEventHandler Elapsed;

        private readonly Timer _timer;

        public CalibrationTimer()
        {
            _timer = new Timer { AutoReset = true };
            _timer.Elapsed += TimerElapsed;
        }

        public double Tick
        {
            get
            {
                return _timer.Interval;
            }
            set
            {
                var enabled = _timer.Enabled;
                if (enabled) Stop();
                _timer.Interval = value;
                if (enabled) Start();
            }
        }

        public void Start()
        {
            _timer.Start();
        }

        public void Stop()
        {
            _timer.Stop();
        }

        private void TimerElapsed(object sender, ElapsedEventArgs e)
        {
            OnElapsed(e);
        }

        private void OnElapsed(ElapsedEventArgs e)
        {
            var handler = Elapsed;
            if (handler != null) handler(this, e);
        }
    }
}
