﻿using System;
using System.Collections.Specialized;
using System.Linq;
using System.Collections.ObjectModel;
using Maze.Input.Wii.Extensions;
using Maze.Input.Wii.Models.Extensions;
using WiimoteLib;

namespace Maze.Input.Wii.Models
{
    public class WiiCollection : ObservableCollection<Wiimote>
    {

        public void PublishInitialCollectionChanged()
        {
            var items = this.ToList();

            var args = new NotifyCollectionChangedEventArgs(NotifyCollectionChangedAction.Add, items, 0);

            OnCollectionChanged(args);
        }

        public void LoadWiimotes()
        {
            var wiimoteCollection = new WiimoteCollection();

            try
            {
                wiimoteCollection.FindAllWiimotes();
            }
            catch (WiimoteException)
            {
                Console.WriteLine("Wiimote error");
                return;
            }
            catch (Exception)
            {
                Console.WriteLine("Unknown error");
                return;
            }

            Clear();

            foreach (var wiimote in wiimoteCollection)
            {
                Add(wiimote);
            }
        }

        protected override void InsertItem(int index, Wiimote wiimote)
        {
            wiimote.WiimoteExtensionChanged += WiimoteExtensionChanged;
            wiimote.Connect();
            wiimote.SetLEDs(this.Count(w => w.GetExtensionType() != ExtensionType.BalanceBoard) + 1);

            base.InsertItem(index, wiimote);
        }

        protected override void RemoveItem(int index)
        {
            var wiimote = this[index];

            wiimote.Disconnect();
            wiimote.WiimoteExtensionChanged -= WiimoteExtensionChanged;

            base.RemoveItem(index);
        }

        protected override void ClearItems()
        {
            foreach (var wiimote in this)
            {
                wiimote.Disconnect();
            }
            base.ClearItems();
        }

        private static void WiimoteExtensionChanged(object sender, WiimoteExtensionChangedEventArgs e)
        {
            var wiimote = (Wiimote)sender;

            if (e.Inserted)
            {
                if (e.ExtensionType == ExtensionType.BalanceBoard)
                {
                    wiimote.SetLEDs(1);
                    wiimote.SetReportType(InputReport.ButtonsExtension, true);
                    return;
                }
                if (e.ExtensionType == ExtensionType.MotionPlus)
                {
                    wiimote.SetMotionPlus(new MotionPlus(wiimote));
                }
            }
            else
            {
                if (e.ExtensionType == ExtensionType.MotionPlus)
                {
                    wiimote.SetMotionPlus(null);
                }
            }
            wiimote.SetReportType(e.Inserted ? InputReport.ExtensionAccel : InputReport.IRAccel, true);
        }
    }
}