﻿using System;
using System.Linq;
using Maze.Input.Wii.Extensions;
using Maze.Input.Wii.Models;
using Maze.Input.Wii.Models.Extensions;
using WiimoteLib;

namespace Maze.Input.Wii
{
    public class WiiDetecter
    {
        private static WiiDetecter _instance;

        protected WiiDetecter()
        {
            Wiimotes = new WiiCollection();
        }

        public static WiiDetecter GetInstance()
        {
            return _instance ?? (_instance = new WiiDetecter());
        }

        public WiiCollection Wiimotes
        {
            get;
            private set;
        }

        public int WiimoteCount
        {
            get
            {
                return Wiimotes.Count;
            }
        }

        public BalanceBoard BalanceBoard
        {
            get
            {
                foreach (var wiimote in Wiimotes.Where(wiimote => wiimote.HasExtension() && wiimote.GetExtensionType() == ExtensionType.BalanceBoard))
                {
                    return wiimote.GetBalanceBoard();
                }
                throw new Exception("No wiimote with BalanceBoard-extension detected.");
            }
        }

        public bool HasMotionPlus()
        {
            return Wiimotes.Any(wiimote => wiimote.HasExtension() && wiimote.GetExtensionType() == ExtensionType.MotionPlus);
        }

        public bool HasBalanceBoard()
        {
            return Wiimotes.Any(wiimote => wiimote.HasExtension() && wiimote.GetExtensionType() == ExtensionType.BalanceBoard);
        }

        public void LoadWiimotes()
        {
            Wiimotes.Clear();
            try
            {
                Wiimotes.LoadWiimotes();
            }
            catch (WiimoteNotFoundException)
            {
                Console.WriteLine("No Wiimotes found.");
            }
        }

        public void Destroy()
        {
            
        }
    }
}
