﻿using System;
using System.Collections.Generic;
using System.Linq;
using Maze.Input.Wrapper;

namespace Maze.Input
{
    class WrapperFactory
    {
        private readonly Dictionary<Input, Type> _inputToTypeTranslator;
        private readonly WrapperList _wrapperList;

        public WrapperFactory()
        {
            _inputToTypeTranslator = new Dictionary<Input, Type>();
            _wrapperList = new WrapperList();

            InitializeInputToTypeTranslator();
        }

        public ReadOnlyWrapperList GetWrappers(Input inputs)
        {
            var inputTypes = Enum.GetValues(typeof(Input)).Cast<Input>();
            foreach (var input in inputTypes.Where(input => inputs.HasFlag(input)))
            {
                var translatedType = TranslateInput(input);
                if (_wrapperList.HasWrapper(translatedType)) continue;

                _wrapperList.AddWrapper(GetWrapper(input));
            }
            return new ReadOnlyWrapperList(_wrapperList);
        }

        private BaseInputWrapper GetWrapper(Input input)
        {
            switch (input)
            {
                case Input.Keyboard:
                    return new KeyboardWrapper();
                case Input.Wii:
                    return new WiiWrapper();
                case Input.Kinect:
                    return new KinectWrapper();
                case Input.Move:
                    return new MoveWrapper();
                default:
                    return new FakeWrapper();
            }
        }

        private Type TranslateInput(Input input)
        {
            return _inputToTypeTranslator.ContainsKey(input) ? _inputToTypeTranslator[input] : typeof (FakeWrapper);
        }

        private void InitializeInputToTypeTranslator()
        {
            _inputToTypeTranslator.Add(Input.Keyboard, typeof(KeyboardWrapper));
            _inputToTypeTranslator.Add(Input.Wii, typeof(WiiWrapper));
            _inputToTypeTranslator.Add(Input.Kinect, typeof(KinectWrapper));
            _inputToTypeTranslator.Add(Input.Move, typeof(MoveWrapper));
        }
    }
}
