﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using Maze.Input.Wrapper;

namespace Maze.Input
{
    public class ReadOnlyWrapperList : ReadOnlyCollection<IInputWrapper>
    {
        private readonly WrapperList _list ;

        public ReadOnlyWrapperList(WrapperList list) : base(list)
        {
            _list = list;
        }

        public List<PollableInputWrapper> PollableWrappers
        {
            get
            {
                return this.OfType<PollableInputWrapper>().ToList();
            }
        }

        public List<UnpollableInputWrapper> UnpollableWrappers
        {
            get
            {
                return this.OfType<UnpollableInputWrapper>().ToList();
            }
        }

        public bool HasWrapper(Type type)
        {
            return _list.HasWrapper(type);
        }

        public IInputWrapper GetWrapper(Type type)
        {
            return _list.GetWrapper(type);
        }
    }

    public class WrapperList : List<IInputWrapper>
    {
        public bool HasWrapper(Type type)
        {
            return ContainsType(type);
        }

        public void AddWrapper(IInputWrapper wrapper)
        {
            if (!ContainsWrapper(wrapper))
            {
                Add(wrapper);
            }
        }

        public IInputWrapper GetWrapper(Type type)
        {
            return this.Where((item) => item.GetType() == type).First();
        }

        private bool ContainsWrapper(IInputWrapper wrapper)
        {
            return ContainsType(wrapper.GetType());
        }

        private bool ContainsType(Type type)
        {
            return this.Any(item => item.GetType() == type);
        }
    }
}
