﻿using System;
using System.Collections.Generic;
using Maze.Models;

namespace Maze
{
    public class LevelMap
    {
        protected readonly List<LevelPath> Levels;

        public LevelMap()
        {
            Levels = new List<LevelPath>();

            InitializeLevelMap();
        }

        public int GetLevelNumber(Level level)
        {
            if (Levels.Contains(level.LevelPath))
            {
                return level.LevelPath.LevelNumber;
            }
            throw new Exception("Level does not exist");
        }

        public LevelPath GetLevel(int levelNumber)
        {
            var levelToLoad = levelNumber - 1;
            if (levelToLoad >= 0 && levelToLoad < Levels.Count)
            {
                return Levels[levelToLoad];
            }
            throw new Exception("Level does not exist");
        }

        public LevelPath GetNextLevel(Level currentLevel)
        {
            var currentLevelIndex = Levels.IndexOf(currentLevel.LevelPath);
            return Levels[currentLevelIndex + 1];
        }

        public bool HasMoreLevels(Level currentLevel)
        {
            var currentLevelIndex = Levels.IndexOf(currentLevel.LevelPath);
            return (currentLevelIndex + 1) < Levels.Count;
        }

        private void InitializeLevelMap()
        {
            switch (Globals.GameDifficulty)
            {
                case Options.Difficulty.Easy:
                    break;
                case Options.Difficulty.Normal:
                    AddLevelPath("Level1", "Easy"); // Easy
                    AddLevelPath("Level3", "Easy"); // Medium
                    AddLevelPath("Level2", "Easy"); // Hard
                    break;
                case Options.Difficulty.Hard:
                    break;
            }
//            AddLevelPath("Level4", null);
        }

        private void AddLevelPath(string name, string path)
        {
            Levels.Add(new LevelPath(name, path, Levels.Count + 1));
        }
    }
}
