using System.Linq;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Utilities;
using Maze.Extensions;
using Maze.Input;
using Maze.Input.Wrapper;
using Maze.Screens;
using Maze.Screens.Menu;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Input;

namespace Maze
{
    public class Maze : Game
    {
        private readonly InputManager _inputManager;
        private readonly GraphicsDeviceManager _graphics;

        public Maze()
        {
            _inputManager = InputManager.GetInstance();

            _graphics = new GraphicsDeviceManager(this);
            _graphics.PreferredBackBufferHeight = Globals.ViewportHeight;
            _graphics.PreferredBackBufferWidth = Globals.ViewportWidth;
            Content.RootDirectory = "Content";
			
			BackStack<string> bs = new BackStack<string>();
			bs.Current = string.Empty;
        }

        public static void ExitGame()
        {
            InputManager.GetInstance().Destroy();
            
            ScreenManager.CurrentScreen.Destroy();
            
            FlatRedBallServices.Game.Exit();
        }

        protected override void Initialize()
        {
            Window.AllowUserResizing = Globals.AllowUserResizing;
            Window.Title = Globals.WindowTitle;
            
            var graphicsOptions = InitializeGraphicsOptions();
            FlatRedBallServices.InitializeFlatRedBall(this, _graphics, graphicsOptions);

            // Disable Post-Processing
            Renderer.UseRenderTargets = false;

            IsMouseVisible = Globals.IsMouseVisible;

            SpriteManager.Camera.SetupCamera();
            _inputManager.DetectInput();

            ScreenManager.Start(typeof(MainMenu).FullName);
//            ScreenManager.Start(typeof(GameScreen).FullName);

            base.Initialize();
        }


        protected override void Update(GameTime gameTime)
        {
            if (_inputManager.KeyPushed(Keys.Escape))
            {
                ExitGame();
                return;
            }

            if (_inputManager.KeyPushed(Keys.F1))
            {
                _inputManager.DetectInput();
            }

            FlatRedBallServices.Update(gameTime);
            ScreenManager.Activity();
        }

        protected override void Draw(GameTime gameTime)
        {
            FlatRedBallServices.Draw();
        }

        private static GraphicsOptions InitializeGraphicsOptions()
        {
            var graphicsOptions = new GraphicsOptions();

            graphicsOptions.SuspendDeviceReset();

            graphicsOptions.IsFullScreen = Globals.IsFullScreen;
            graphicsOptions.SetResolution(Globals.ViewportWidth, Globals.ViewportHeight);
            graphicsOptions.UseMultiSampling = true;

            graphicsOptions.ResumeDeviceReset();
            return graphicsOptions;
        }
    }
}
