﻿using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math.Geometry;
using Microsoft.Xna.Framework.Graphics;

namespace Maze.Models
{
    public abstract class CollisionEntity : Entity
    {
        protected Texture2D Texture;

        protected CollisionEntity(Layer layer, string contentManager) : base(contentManager)
        {
            Initialize(false);
            AddToManager(layer);
        }

        public Sprite Sprite
        {
            get;
            protected set;
        }

        public Circle Collision
        {
            get;
            protected set;
        }

        public bool Visible
        {
            get
            {
                return Sprite.Visible;
            }
            set
            {
                Sprite.Visible = value;
            }
        }

        public override void AddToManager(Layer layer)
        {
            base.AddToManager(layer);

            InitializeSprite(layer);
            InitializeCollision(layer);
        }

        public override void Destroy()
        {
            SpriteManager.RemoveSprite(Sprite);
            ShapeManager.Remove(Collision);
            base.Destroy();
        }

        public void Scale(int scale)
        {
            Sprite.ScaleX = scale;
            Sprite.ScaleY = scale;
            Collision.Radius = scale;
        }

        public void PositionSpriteUsingPlaceholderAsReference(Sprite placeholder)
        {
            Position = placeholder.Position;
            Visible = true;
        }

        protected virtual void InitializeSprite(Layer layer)
        {
            Sprite = SpriteManager.AddSprite(Texture, layer);
            Sprite.PixelSize = .5f;
            Sprite.AttachTo(this, false);
        }

        protected virtual void InitializeCollision(Layer layer)
        {
            Collision = ShapeManager.AddCircle();
            Collision.Radius = Sprite.ScaleX;
            Collision.AttachTo(this, false);
            ShapeManager.AddToLayer(Collision, layer);

            Collision.Visible = Globals.IsCollisionsVisible;
        }
    }
}
