﻿using System.Collections.Specialized;
using System.Drawing.Imaging;
using System.IO;
using System.Linq;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;
using Maze.Input.Move;
using Maze.Input.Move.Models;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using PSMoveSharp;

namespace Maze.Models.HUDs
{
    public class MoveHUD : BaseHUD
    {
        private readonly MoveDetecter _moveDetecter;
        private readonly PositionedObjectList<Sprite> _sprites;
        private Sprite _videoSprite;

        public MoveHUD(string contentManagerName) : base(contentManagerName)
        {
            _moveDetecter = MoveDetecter.GetInstance();
            _sprites = new PositionedObjectList<Sprite>();

            Initialize(false);
            AddToManager(Camera.Layer);
            
            SubscribeToEvents();
        }

        protected Text MovePosition
        {
            get;
            set;
        }

        protected Text MoveOrientation
        {
            get;
            set;
        }

        protected Text ConnectedMoves
        {
            get;
            set;
        }

        protected Text Delay
        {
            get;
            set;
        }

        public override void AddToManager(Layer layer)
        {
            InitializeMovePositionDisplay(layer);
            InitializeMoveOrientationDisplay(layer);
            InitializeConnectedMovesDisplay(layer);
            InitializeDelayDisplay(layer);
            
//            InitializeVideoOverlay(layer);

            base.AddToManager(layer);
        }

        public override void Activity()
        {
            ConnectedMoves.DisplayText = "Connected Moves: " + _moveDetecter.MoveCount;
            Delay.DisplayText = "Delay: " + _moveDetecter.Delay + " ms";
            
            base.Activity();
        }

        public override void Destroy()
        {
            UnsubscribeFromEvents();
            SpriteManager.RemoveSpriteList(_sprites);
            base.Destroy();
        }

        private void SubscribeToEvents()
        {
            _moveDetecter.Moves.CollectionChanged += OnMoveCollectionChanged;

//            MoveClient.GetInstance().CameraFrameStateUpdated += OnCameraFrameStateUpdated;

            foreach (var move in _moveDetecter.Moves)
            {
                SubscribeToMove(move);
            }
        }

        private void UnsubscribeFromEvents()
        {
            _moveDetecter.Moves.CollectionChanged -= OnMoveCollectionChanged;

//            MoveClient.GetInstance().CameraFrameStateUpdated -= OnCameraFrameStateUpdated;

            foreach (var move in _moveDetecter.Moves)
            {
                UnsubscribeFromMove(move);
            }
        }

        private void SubscribeToMove(Move move)
        {
            move.Activity += OnMoveActivity;
        }

        private void UnsubscribeFromMove(Move move)
        {
            move.Activity -= OnMoveActivity;
        }

        private void OnMoveActivity(Move move)
        {
            if (move.MoveID != MoveClient.MoveID.First) return;

            MovePosition.DisplayText = "Position: " + move.Position;
            MoveOrientation.DisplayText = "Orientation: " + move.Orientation;
        }

//        private void OnCameraFrameStateUpdated(PSMoveSharpCameraFrameState state)
//        {
//            state.camera_frame_state_rwl.AcquireReaderLock(-1);
//            
//            var dummyState = new PSMoveSharpState();
//            var image = state.GetCameraFrameAndState(ref dummyState);
//
//            using (var stream = new MemoryStream())
//            {
//                image.Save(stream, ImageFormat.Jpeg);
//                stream.Seek(0, SeekOrigin.Begin);
//
//                _videoSprite.Texture = Texture2D.FromStream(FlatRedBallServices.GraphicsDevice, stream);
//            }
//
//            state.camera_frame_state_rwl.ReleaseReaderLock(); 
//        }

        private void OnMoveCollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
        {
            if (e.Action == NotifyCollectionChangedAction.Reset || e.Action == NotifyCollectionChangedAction.Move)
                return;

            if (e.NewItems != null)
            {
                foreach (var move in from object item in e.NewItems select item as Move)
                {
                    SubscribeToMove(move);
                }
            }

            if (e.OldItems != null)
            {
                foreach (var move in from object item in e.OldItems select item as Move)
                {
                    UnsubscribeFromMove(move);
                }
            }
        }

        private void InitializeMovePositionDisplay(Layer layer)
        {
            MovePosition = CreateText(new Vector3(10, 50, 0), layer);
        }

        private void InitializeMoveOrientationDisplay(Layer layer)
        {
            MoveOrientation = CreateText(new Vector3(10, 90, 0), layer);
        }

        private void InitializeConnectedMovesDisplay(Layer layer)
        {
            ConnectedMoves = CreateText(new Vector3(10, 130, 0), layer);
        }

        private void InitializeDelayDisplay(Layer layer)
        {
            Delay = CreateText(new Vector3(10, 170, 0), layer);
        }

//        private void InitializeVideoOverlay(Layer layer)
//        {
//            _videoSprite = new Sprite { ScaleX = 640 / 2, ScaleY = 480 / 2 };
//            _videoSprite.Position = new Vector3(_videoSprite.ScaleX, _videoSprite.ScaleY, 0);
//            SpriteManager.AddSprite(_videoSprite);
//            SpriteManager.AddToLayer(_videoSprite, layer);
//            _sprites.Add(_videoSprite);
//        }
    }
}
