﻿using System.Linq;
using FlatRedBall.Graphics;
using Maze.Input.Wii;
using Maze.Input.Wii.Extensions;
using Microsoft.Xna.Framework;

namespace Maze.Models.HUDs
{
    class WiiHUD : BaseHUD
    {
        private readonly WiiDetecter _wiiDetecter;

        public WiiHUD(string contentManagerName) : base(contentManagerName)
        {
            _wiiDetecter = WiiDetecter.GetInstance();

            Initialize(false);
            AddToManager(Camera.Layer);
        }

        protected Text RawYawDisplay
        {
            get;
            set;
        }

        protected Text RawPitchDisplay
        {
            get;
            set;
        }

        protected Text RawRollDisplay
        {
            get;
            set;
        }

        protected Text AccelerationDisplay
        {
            get;
            set;
        }

        protected Text RotationDisplay
        {
            get;
            set;
        }

        public override void AddToManager(Layer layer)
        {
            InitializeYawDisplay(layer);
            InitializePitchDisplay(layer);
            InitializeRollDisplay(layer);
            InitializeRotationDisplay(layer);
            InitializeAccelerationDisplay(layer);

            base.AddToManager(layer);
        }

        public override void Activity()
        {
            var wiimote = _wiiDetecter.Wiimotes.Where(w => w.HasMotionPlus()).FirstOrDefault();

            if (wiimote == null)
            {
                return;
            }

            var motionPlus = wiimote.GetMotionPlus();

            var rotation = motionPlus.GetRotation();

            RawYawDisplay.DisplayText = "Yaw: " + motionPlus.GetRawYaw();
            RawPitchDisplay.DisplayText = "Pitch: " + motionPlus.GetRawPitch();
            RawRollDisplay.DisplayText = "Roll: " + motionPlus.GetRawRoll();
            AccelerationDisplay.DisplayText = "Acceleration: " + motionPlus.Acceleration;
            RotationDisplay.DisplayText = "Angle: " + rotation;

            base.Activity();
        }

        private void InitializeRotationDisplay(Layer layer)
        {
            RotationDisplay = TextManager.AddText("", layer);
            RotationDisplay.Font = Font;
            RotationDisplay.Position = new Vector3(50, 300, 0);

            RotationDisplay.Red = 1f;
            RotationDisplay.Green = 1f;
            RotationDisplay.Blue = 1f;

            RotationDisplay.SetPixelPerfectScale(layer);

            Texts.Add(RotationDisplay);
        }

        private void InitializeYawDisplay(Layer layer)
        {
            RawYawDisplay = TextManager.AddText("", layer);
            RawYawDisplay.Font = Font;
            RawYawDisplay.Position = new Vector3(50, 250, 0);

            RawYawDisplay.Red = 1f;
            RawYawDisplay.Green = 1f;
            RawYawDisplay.Blue = 1f;

            RawYawDisplay.SetPixelPerfectScale(layer);

            Texts.Add(RawYawDisplay);
        }

        private void InitializePitchDisplay(Layer layer)
        {
            RawPitchDisplay = TextManager.AddText("", layer);
            RawPitchDisplay.Font = Font;
            RawPitchDisplay.Position = new Vector3(50, 200, 0);

            RawPitchDisplay.Red = 1f;
            RawPitchDisplay.Green = 1f;
            RawPitchDisplay.Blue = 1f;

            RawPitchDisplay.SetPixelPerfectScale(layer);

            Texts.Add(RawPitchDisplay);
        }

        private void InitializeRollDisplay(Layer layer)
        {
            RawRollDisplay = TextManager.AddText("", layer);
            RawRollDisplay.Font = Font;
            RawRollDisplay.Position = new Vector3(50, 150, 0);

            RawRollDisplay.Red = 1f;
            RawRollDisplay.Green = 1f;
            RawRollDisplay.Blue = 1f;

            RawRollDisplay.SetPixelPerfectScale(layer);

            Texts.Add(RawRollDisplay);
        }

        private void InitializeAccelerationDisplay(Layer layer)
        {
            AccelerationDisplay = TextManager.AddText("", layer);
            AccelerationDisplay.Font = Font;
            AccelerationDisplay.Position = new Vector3(50, 100, 0);

            AccelerationDisplay.Red = 1f;
            AccelerationDisplay.Green = 1f;
            AccelerationDisplay.Blue = 1f;

            AccelerationDisplay.SetPixelPerfectScale(layer);

            Texts.Add(AccelerationDisplay);
        }
    }
}
