﻿using FlatRedBall;
using Maze.Input;
using Microsoft.Xna.Framework.Graphics;

namespace Maze.Screens
{
    public class BaseScreen : Screen
    {
        protected InputManager InputManager;
        private Texture2D _backgroundTexture;

        public BaseScreen(string contentManagerName) : base(contentManagerName)
        {
            InputManager = InputManager.GetInstance();
        }

        public Camera Camera
        {
            get { return SpriteManager.Camera; }
        }

        public Sprite Background
        {
            get;
            private set;
        }

        public override void Initialize(bool addToManagers)
        {
            if (addToManagers)
            {
                AddToManagers();
            }
        }

        public override void AddToManagers()
        {
            InitializeBackground();
        }

        public override void Destroy()
        {
            SpriteManager.RemoveSprite(Background);
            base.Destroy();
        }

        protected void LoadBackground(string path)
        {
            _backgroundTexture = LoadTexture(path);
        }

        protected Texture2D LoadTexture(string path, bool useContentPipeline = true)
        {
            path = useContentPipeline ? @"Content\" + path : path;
            return FlatRedBallServices.Load<Texture2D>(path, ContentManagerName);
        }

        protected void InitializeBackground()
        {
            Background = SpriteManager.AddSprite(_backgroundTexture);
            Background.PixelSize = .5f;
            Background.X = Globals.ViewportWidth * .5f;
            Background.Y = Globals.ViewportHeight * .5f;
            mSprites.Add(Background);
        }

        protected void EnableFiltering(bool enabled)
        {
            if (enabled)
            {
                FlatRedBallServices.GraphicsOptions.TextureFilter = TextureFilter.Linear;
                return;
            }
            FlatRedBallServices.GraphicsOptions.TextureFilter = TextureFilter.Point;
        }
    }
}
