﻿using System;
using Maze.Models.Menu;
using Microsoft.Xna.Framework;
using Game = Maze.Models.Game;

namespace Maze.Screens.Menu
{
    public class MainMenu : MenuScreen
    {
        public MainMenu() : base("MainMenu")
        {
        }

        public Button NewGameButton
        {
            get; 
            private set;
        }

        public Button OptionsButton
        {
            get;
            private set;
        }

        public Button ExitGameButton
        {
            get; 
            private set;
        }

        public override void AddToManagers()
        {
            base.AddToManagers();

            InitializeNewGameButton();
            InitializeOptionsButton();
            InitializeExitGameButton();
        }

        private void InitializeNewGameButton()
        {
            NewGameButton = new Button("New Game", ContentManagerName);
            NewGameButton.PositionButton(Globals.CenterOfScreen, new Vector2(0, 100));
            Buttons.Add(NewGameButton);

            NewGameButton.Clicked += NewGameButtonClicked;
        }

        private void InitializeOptionsButton()
        {
            OptionsButton = new Button("Options", ContentManagerName);
            OptionsButton.PositionButton(Globals.CenterOfScreen);
            Buttons.Add(OptionsButton);

            OptionsButton.Clicked += OptionsButtonClicked;
        }

        private void InitializeExitGameButton()
        {
            ExitGameButton = new Button("Exit Game", ContentManagerName);
            ExitGameButton.PositionButton(Globals.CenterOfScreen, new Vector2(0, -100));
            Buttons.Add(ExitGameButton);

            ExitGameButton.Clicked += ExitGameButtonClicked;
        }

        private void NewGameButtonClicked(object sender, EventArgs e)
        {
            Game.ResetInstance();
            IsActivityFinished = true;
            MoveToScreen(typeof(GameScreen).FullName);
        }

        private void OptionsButtonClicked(object sender, EventArgs e)
        {
            MoveToScreen(typeof(OptionsMenu).FullName);
        }

        private void ExitGameButtonClicked(object sender, EventArgs e)
        {
            Maze.ExitGame();
        }
    }
}
