﻿using System.Linq;
using Maze.Models.Menu;
using Microsoft.Xna.Framework.Input;

namespace Maze.Screens.Menu
{
    public abstract class MenuScreen : BaseScreen
    {
        protected const int ButtonGap = 100;

        protected ButtonCollection Buttons;

        protected MenuScreen(string contentManagerName) : base(contentManagerName)
        {
            Buttons = new ButtonCollection();
        }

        public override void Initialize(bool addToManager)
        {
            LoadBackground(@"Menu\Background");

            base.Initialize(addToManager);
        }

        public override void Activity(bool firstTimeCalled)
        {
            base.Activity(firstTimeCalled);

            if (firstTimeCalled && Buttons.Count > 0)
            {
                Buttons.SelectButton(0);
            }

            MouseActivity();
            KeyboardActivity();

            for (int i = Buttons.Count - 1; i > -1; i--)
            {
                Buttons[i].Activity();
            }
        }

        public override void Destroy()
        {
            while (Buttons.Count != 0)
            {
                Buttons.Last.Destroy();
            }
            base.Destroy();
        }

        private void MouseActivity()
        {
            foreach (var button in Buttons.Where(button => InputManager.Mouse.IsOn(button.Collision)))
            {
                Buttons.SelectButton(button);
                if (IsMouseClicked())
                {
                    button.WasClicked();
                }
            }
        }

        private void KeyboardActivity()
        {
            if (InputManager.KeyPushed(Keys.Up))
            {
                Buttons.SelectPreviousButton();
            }
            if (InputManager.KeyPushed(Keys.Down))
            {
                Buttons.SelectNextButton();
            }
            if (InputManager.KeyPushed(Keys.Enter))
            {
                Buttons.SelectedButton.WasClicked();
            }
        }

        private bool IsMouseClicked()
        {
            return InputManager.Mouse.ButtonPushed(FlatRedBall.Input.Mouse.MouseButtons.LeftButton);
        }
    }
}
