﻿using System;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;
using Maze.Models;
using Maze.Models.Menu;
using Microsoft.Xna.Framework;

namespace Maze.Screens.Menu
{
    public class OptionsMenu : MenuScreen
    {
        private PositionedObjectList<Text> _texts;

        public OptionsMenu() : base("OptionsMenu")
        {
        }

        public Options.Difficulty Difficulty
        {
            get
            {
                return Options.GameDifficulty;
            }
        }

        public Text GameDifficultyText
        {
            get;
            private set;
        }

        public Button GameDifficultyButton
        {
            get; 
            private set;
        }

        public Button BackButton
        {
            get; 
            private set;
        }

        public override void Initialize(bool addToManagers)
        {
            _texts = new PositionedObjectList<Text>();

            base.Initialize(addToManagers);
        }

        public override void AddToManagers()
        {
            base.AddToManagers();

            InitializeDifficulty();
            InitializeBackButton();
        }

        public override void Destroy()
        {
            TextManager.RemoveText(_texts);
            base.Destroy();
        }

        private void InitializeDifficulty()
        {
            InitializeDifficultyText();
            InitializeDifficultyButton();
        }
        
        private void InitializeDifficultyText()
        {
            GameDifficultyText = TextManager.AddText("Game Difficulty:");
            GameDifficultyText.Position = Globals.PositionUsingOffset(new Vector2(Globals.CenterX, .8f * Globals.ViewportHeight), new Vector2(-400, 0));

            GameDifficultyText.Red = 0f;
            GameDifficultyText.Green = 0f;
            GameDifficultyText.Blue = 0f;

            GameDifficultyText.Font = new BitmapFont(
                "Content/Fonts/calibri_48",
                "Content/Fonts/calibri_48.fnt",
                FlatRedBallServices.GlobalContentManager);
            GameDifficultyText.SetPixelPerfectScale(Camera);
            GameDifficultyText.RelativeY = .1f;

            _texts.Add(GameDifficultyText);
        }

        private void InitializeDifficultyButton()
        {
            GameDifficultyButton = new Button(Difficulty.ToString(), ContentManagerName);
            GameDifficultyButton.PositionButton(new Vector2(Globals.CenterX, .8f * Globals.ViewportHeight), new Vector2(300, 0));
            Buttons.Add(GameDifficultyButton);

            GameDifficultyButton.Clicked += DifficultyButtonClicked;
        }

        private void InitializeBackButton()
        {
            BackButton = new Button("<- Back", ContentManagerName);
            BackButton.PositionButton(new Vector2(200, 80));
            Buttons.Add(BackButton);

            BackButton.Clicked += BackButtonClicked;
        }

        private void DifficultyButtonClicked(object sender, EventArgs e)
        {
            ChangeDifficulty();
        }

        private void BackButtonClicked(object sender, EventArgs e)
        {
            IsActivityFinished = true;
            MoveToScreen(typeof(MainMenu).FullName);
        }

        private void ChangeDifficulty()
        {
            Options.ChangeDifficulty();
            ChangeDifficultyText();
        }

        private void ChangeDifficultyText()
        {
            GameDifficultyButton.Text.DisplayText = Difficulty.ToString();
        }
    }
}
