﻿using System;
using System.Timers;
using FlatRedBall.Graphics;
using Maze.Extensions;
using Microsoft.Xna.Framework;

namespace Maze.Screens.Pause
{
    public class LevelCompletedScreen : PausedScreen
    {
        public event EventHandler GameCompleted;
        public event EventHandler LevelFinished;

        private Timer _timer;
        private bool _shouldDoActivity;
        private bool _shouldGameFinish;

        public LevelCompletedScreen() : base("LevelCompleted")
        {
            _shouldDoActivity = true;
            _shouldGameFinish = false;
        }

        private Text LevelCompleteText
        {
            get;
            set;
        }

        private Text LoadingLevelText
        {
            get;
            set;
        }

        public override void AddToManagers()
        {
            base.AddToManagers();

            InitializeLevelCompleteText(Layer);
            InitializeLoadingLevelText(Layer);

            HandleGameProgression();

            _timer = new Timer(1000) { AutoReset = false };
            _timer.Elapsed += TimerElapsed;
        }

        public override void Activity(bool firstTimeCalled)
        {
            if (firstTimeCalled ||!_shouldDoActivity) return;
            
            if (_shouldGameFinish)
            {
                GameFinished();
            }
            _timer.Start();

            _shouldDoActivity = false;
        }

        private void HandleGameProgression()
        {
            if (Game.HasMoreLevels())
            {
                Game.IncrementLevel();
                Game.AddScore(1000);
            }
            else
            {
                _shouldGameFinish = true;
            }
        }

        private void GameFinished()
        {
            IsActivityFinished = true;
            OnEvent(GameCompleted, EventArgs.Empty);
        }

        private void TimerElapsed(object sender, ElapsedEventArgs e)
        {
            IsActivityFinished = true;
            OnEvent(LevelFinished, EventArgs.Empty);
        }

        private void InitializeLevelCompleteText(Layer layer)
        {
            LevelCompleteText = TextManager.AddText("Level " + Game.LevelNumber + " complete!", layer);
            LevelCompleteText.Font = Font;
            LevelCompleteText.SetPixelPerfectScale(layer);

            LevelCompleteText.Position = new Vector3(Globals.CenterX - LevelCompleteText.TextCenter(), Globals.ViewportHeight - 200, ZPosition);
            LevelCompleteText.Red = 1f;
            LevelCompleteText.Green = 1f;
            LevelCompleteText.Blue = 1f;

            Texts.Add(LevelCompleteText);
        }

        private void InitializeLoadingLevelText(Layer layer)
        {
            LoadingLevelText = TextManager.AddText("Loading next level...", layer);
            LoadingLevelText.Font = Font;
            LoadingLevelText.SetPixelPerfectScale(layer);

            LoadingLevelText.Position = new Vector3(Globals.CenterX - LoadingLevelText.TextCenter(), Globals.CenterY, ZPosition);
            LoadingLevelText.Red = 1f;
            LoadingLevelText.Green = 1f;
            LoadingLevelText.Blue = 1f;

            Texts.Add(LoadingLevelText);
        }
    }
}
