﻿using System;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;
using Maze.Models;

namespace Maze.Screens.Pause
{
    public abstract class PausedScreen : BaseScreen
    {
        protected const int ZPosition = 1;

        protected PositionedObjectList<Text> Texts;

        protected PausedScreen(string contentManagerName) : base(contentManagerName)
        {
        }

        protected Game Game
        {
            get
            {
                return Game.GetInstance();
            }
        }

        protected BitmapFont Font
        {
            get;
            set;
        }

        public override void Initialize(bool addToManagers)
        {
            LoadBackground(@"PauseBackground");

            Texts = new PositionedObjectList<Text>();

            Font = new BitmapFont(
                "Content/Fonts/calibri_48",
                "Content/Fonts/calibri_48.fnt",
                FlatRedBallServices.GlobalContentManager);

            base.Initialize(addToManagers);
        }

        public override void AddToManagers()
        {
            base.AddToManagers();

            Background.Z = ZPosition;
            SpriteManager.AddToLayer(Background, Layer);
        }

        public override void Destroy()
        {
            TextManager.RemoveText(Texts);
            SpriteManager.RemoveSprite(Background);
        }

        protected void OnEvent(EventHandler handler, EventArgs e)
        {
            if (handler != null) handler(this, e);
        }
    }
}
