﻿using Maze.Events;
using Maze.Helpers;
using Maze.Models;

namespace Maze.Screens
{
    public class RotatingCameraScreen : BaseScreen
    {
        private readonly RotatingCamera _camera;
        private readonly RotateCameraEvent _rotateCameraEvent;
        private readonly CameraRotatedEvent _cameraRotatedEvent;

        public RotatingCameraScreen(string contentManagerName) : base(contentManagerName)
        {
            _camera = CameraHelper.CreateRotatingCamera(ContentManagerName);

            _rotateCameraEvent = new RotateCameraEvent();
            _cameraRotatedEvent = new CameraRotatedEvent();
            SubscribeToEvents();
        }

        public RotatingCamera RotatingCamera
        {
            get
            {
                return _camera;
            }
        }

        public DirectionHelper.CameraAngle CameraAngle
        {
            get
            {
                return _camera.CameraAngle;
            }
        }

        public override void Activity(bool firstTimeCalled)
        {
            RotatingCamera.Activity();

            base.Activity(firstTimeCalled);
        }

        public override void Destroy()
        {
            CameraHelper.DestroyCamera(_camera);
            UnsubscribeFromEvents();

            base.Destroy();
        }

        public void RotateCamera(RotatingCamera.Direction direction)
        {
            RotatingCamera.Rotate(direction);
            _cameraRotatedEvent.Publish(new CameraRotatedEventArgs() { Angle = RotatingCamera.Rotation });
        }

        public void RotateCamera(float angle, bool relativeRotation)
        {
            RotatingCamera.Rotate(angle, relativeRotation);
            _cameraRotatedEvent.Publish(new CameraRotatedEventArgs() { Angle =  RotatingCamera.Rotation });
        }

        private void SubscribeToEvents()
        {
            _rotateCameraEvent.Subscribe(OnRotateCamera);
        }

        private void UnsubscribeFromEvents()
        {
            _rotateCameraEvent.Unsubscribe(OnRotateCamera);
        }

        private void OnRotateCamera(RotateCameraEventArgs args)
        {
            RotateCamera(args.Rotation, !args.Absolute);
        }
    }
}
