using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Graphics.Model;

using FlatRedBall.Input;
using FlatRedBall.Utilities;

using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;
using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;
// Generated Usings
using RiverGame.Screens;
using Matrix = Microsoft.Xna.Framework.Matrix;
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;
using FlatRedBall.ManagedSpriteGroups;
using FlatRedBall.Graphics.Animation;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
#endif

#if FRB_XNA
using Model = Microsoft.Xna.Framework.Graphics.Model;
#endif

namespace RiverGame.Entities
{
	public partial class Frame : PositionedObject, IDestroyable, IVisible
	{
        // This is made global so that static lazy-loaded content can access it.
        public static string ContentManagerName
        {
            get;
            set;
        }

		// Generated Fields
#if DEBUG
static bool HasBeenLoadedWithGlobalContentManager = false;
#endif
public enum VariableState
{
	Uninitialized, //This exists so that the first set call actually does something
	RoundedRed, 
	SquareGray, 
	RoundedGray, 
	SingleRoundedGray
}
static object mLockObject = new object();
static bool mHasRegisteredUnload = false;
static bool IsStaticContentLoaded = false;
private static AnimationChainList AnimationChainListFile;
private static Scene SceneFile;

private SpriteFrame SpriteFrameInstance;
public string CurrentChain
{
	get
	{
		return SpriteFrameInstance.CurrentChainName;
	}
	set
	{
		SpriteFrameInstance.CurrentChainName = value;
	}
}
public float ScaleX
{
	get
	{
		return SpriteFrameInstance.ScaleX;
	}
	set
	{
		SpriteFrameInstance.ScaleX = value;
	}
}
public float ScaleY
{
	get
	{
		return SpriteFrameInstance.ScaleY;
	}
	set
	{
		SpriteFrameInstance.ScaleY = value;
	}
}
public event EventHandler BeforeVisibleSet;
public event EventHandler AfterVisibleSet;
protected bool mVisible = true;
public virtual bool Visible
{
	get
	{
		return mVisible;
	}
	set
	{
		bool hasChanged = value != mVisible;
		if (BeforeVisibleSet != null)
		{
			BeforeVisibleSet(this, null);
		}
		mVisible = value;
		SpriteFrameInstance.Visible = mVisible;
		if (AfterVisibleSet != null)
		{
			AfterVisibleSet(this, null);
		}
	}
}
protected Layer LayerProvidedByContainer = null;

        public Frame(string contentManagerName) :
            this(contentManagerName, true)
        {
        }


        public Frame(string contentManagerName, bool addToManagers) :
			base()
		{
			// Don't delete this:
            ContentManagerName = contentManagerName;
            InitializeEntity(addToManagers);

		}

		protected virtual void InitializeEntity(bool addToManagers)
		{
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			SpriteFrameInstance = SceneFile.SpriteFrames.FindByName("AnimationChainListFile").Clone();
			
			PostInitialize();
			if (addToManagers)
			{
				AddToManagers(null);
			}


		}

// Generated AddToManagers
		public virtual void AddToManagers (Layer layerToAddTo)
		{
			LayerProvidedByContainer = layerToAddTo;
			SpriteManager.AddPositionedObject(this);
			AddToManagersBottomUp(layerToAddTo);
			CustomInitialize();
		}

		public virtual void Activity()
		{
			// Generated Activity
			
			CustomActivity();
			
			// After Custom Activity
		}

		public virtual void Destroy()
		{
			// Generated Destroy
			SpriteManager.RemovePositionedObject(this);
			if (SpriteFrameInstance != null)
			{
				SpriteManager.RemoveSpriteFrame(SpriteFrameInstance);
			}
			


			CustomDestroy();
		}

		// Generated Methods
public virtual void PostInitialize ()
{
	CurrentChain = "RoundedRed";
	ScaleX = 32f;
	ScaleY = 32f;
	X = 0f;
	Y = 0f;
	Z = 0f;
}
public virtual void AddToManagersBottomUp (Layer layerToAddTo)
{
	// We move this back to the origin and unrotate it so that anything attached to it can just use its absolute position
	float oldRotationX = RotationX;
	float oldRotationY = RotationY;
	float oldRotationZ = RotationZ;
	
	float oldX = X;
	float oldY = Y;
	float oldZ = Z;
	
	X = 0;
	Y = 0;
	Z = 0;
	RotationX = 0;
	RotationY = 0;
	RotationZ = 0;
	SpriteManager.AddToLayer(SpriteFrameInstance, layerToAddTo);
	if (SpriteFrameInstance.Parent == null)
	{
		SpriteFrameInstance.AttachTo(this, true);
	}
	X = oldX;
	Y = oldY;
	Z = oldZ;
	RotationX = oldRotationX;
	RotationY = oldRotationY;
	RotationZ = oldRotationZ;
}
public virtual void ConvertToManuallyUpdated ()
{
	this.ForceUpdateDependenciesDeep();
	SpriteManager.ConvertToManuallyUpdated(this);
}
public static void LoadStaticContent (string contentManagerName)
{
	ContentManagerName = contentManagerName;
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	if (IsStaticContentLoaded == false)
	{
		IsStaticContentLoaded = true;
		lock (mLockObject)
		{
			if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
			{
				FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("FrameStaticUnload", UnloadStaticContent);
				mHasRegisteredUnload = true;
			}
		}
		bool registerUnload = false;
		if (!FlatRedBallServices.IsLoaded<AnimationChainList>(@"content/entities/frame/animationchainlistfile.achx", ContentManagerName))
		{
			registerUnload = true;
		}
		AnimationChainListFile = FlatRedBallServices.Load<AnimationChainList>(@"content/entities/frame/animationchainlistfile.achx", ContentManagerName);
		if (!FlatRedBallServices.IsLoaded<Scene>(@"content/entities/frame/scenefile.scnx", ContentManagerName))
		{
			registerUnload = true;
		}
		SceneFile = FlatRedBallServices.Load<Scene>(@"content/entities/frame/scenefile.scnx", ContentManagerName);
		if (registerUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
		{
			lock (mLockObject)
			{
				if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
				{
					FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("FrameStaticUnload", UnloadStaticContent);
					mHasRegisteredUnload = true;
				}
			}
		}
		CustomLoadStaticContent(contentManagerName);
	}
}
public static void UnloadStaticContent ()
{
	IsStaticContentLoaded = false;
	mHasRegisteredUnload = false;
	if (AnimationChainListFile != null)
	{
		AnimationChainListFile= null;
	}
	if (SceneFile != null)
	{
		SceneFile.RemoveFromManagers(ContentManagerName != "Global");
		SceneFile= null;
	}
}
public static object GetStaticMember (string memberName)
{
	switch(memberName)
	{
		case  "AnimationChainListFile":
			return AnimationChainListFile;
		case  "SceneFile":
			return SceneFile;
	}
	return null;
}
static VariableState mLoadingState = VariableState.Uninitialized;
public static VariableState LoadingState
{
	get
	{
		return mLoadingState;
	}
	set
	{
		mLoadingState = value;
	}
}
VariableState mCurrentState = VariableState.Uninitialized;
public VariableState CurrentState
{
	get
	{
		return mCurrentState;
	}
	set
	{
		mCurrentState = value;
		switch(mCurrentState)
		{
			case  VariableState.RoundedRed:
				CurrentChain = "RoundedRed";
				break;
			case  VariableState.SquareGray:
				CurrentChain = "SquareGray";
				break;
			case  VariableState.RoundedGray:
				CurrentChain = "RoundedGray";
				break;
			case  VariableState.SingleRoundedGray:
				CurrentChain = "SingleRoundedGray";
				break;
		}
	}
}
public MethodInstruction<Frame> InterpolateToState (VariableState stateToInterpolateTo, double secondsToTake)
{
	switch(stateToInterpolateTo)
	{
		case  VariableState.RoundedRed:
			break;
		case  VariableState.SquareGray:
			break;
		case  VariableState.RoundedGray:
			break;
		case  VariableState.SingleRoundedGray:
			break;
	}
	var instruction = new MethodInstruction<Frame>(this, "StopStateInterpolation", new object[]{stateToInterpolateTo}, TimeManager.CurrentTime + secondsToTake);
	this.Instructions.Add(instruction);
	return instruction;
}
public void StopStateInterpolation (VariableState stateToStop)
{
	switch(stateToStop)
	{
		case  VariableState.RoundedRed:
			break;
		case  VariableState.SquareGray:
			break;
		case  VariableState.RoundedGray:
			break;
		case  VariableState.SingleRoundedGray:
			break;
	}
	CurrentState = stateToStop;
}
public void InterpolateBetween (VariableState firstState, VariableState secondState, float interpolationValue)
{
	switch(firstState)
	{
		case  VariableState.RoundedRed:
			break;
		case  VariableState.SquareGray:
			break;
		case  VariableState.RoundedGray:
			break;
		case  VariableState.SingleRoundedGray:
			break;
	}
	switch(secondState)
	{
		case  VariableState.RoundedRed:
			break;
		case  VariableState.SquareGray:
			break;
		case  VariableState.RoundedGray:
			break;
		case  VariableState.SingleRoundedGray:
			break;
	}
}
object GetMember (string memberName)
{
	switch(memberName)
	{
	}
	return null;
}

    }
	
	
	// Extra classes
	public static class FrameExtensionMethods
	{
		public static void SetVisible (this PositionedObjectList<Frame> list, bool value)
		{
			int count = list.Count;
			for (int i = 0; i < count; i++)
			{
				list[i].Visible = value;
			}
		}
	}
	
}
