using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Graphics.Model;

using FlatRedBall.Input;
using FlatRedBall.Utilities;

using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;
using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;
// Generated Usings
using RiverGame.Screens;
using Matrix = Microsoft.Xna.Framework.Matrix;
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
#endif

#if FRB_XNA
using Model = Microsoft.Xna.Framework.Graphics.Model;
#endif

namespace RiverGame.Entities
{
	public partial class Options : PositionedObject, IDestroyable, IVisible
	{
        // This is made global so that static lazy-loaded content can access it.
        public static string ContentManagerName
        {
            get;
            set;
        }

		// Generated Fields
#if DEBUG
static bool HasBeenLoadedWithGlobalContentManager = false;
#endif
static object mLockObject = new object();
static bool mHasRegisteredUnload = false;
static bool IsStaticContentLoaded = false;

private RiverGame.Entities.Frame FrameInstance;
private RiverGame.Entities.CheckBox KinectCheckBox;
private RiverGame.Entities.CheckBox WiiCheckBox;
private RiverGame.Entities.CheckBox MoveCheckBox;
private RiverGame.Entities.Button BackButton;
public RiverGame.Entities.CheckBox.VariableState KinectCheckBoxCurrentState
{
	get
	{
		return KinectCheckBox.CurrentState;
	}
	set
	{
		KinectCheckBox.CurrentState = value;
	}
}
public RiverGame.Entities.CheckBox.VariableState WiiCheckBoxCurrentState
{
	get
	{
		return WiiCheckBox.CurrentState;
	}
	set
	{
		WiiCheckBox.CurrentState = value;
	}
}
public RiverGame.Entities.CheckBox.VariableState MoveCheckBoxCurrentState
{
	get
	{
		return MoveCheckBox.CurrentState;
	}
	set
	{
		MoveCheckBox.CurrentState = value;
	}
}
public event FlatRedBall.Gui.WindowEvent BackButtonClick;
public event FlatRedBall.Gui.WindowEvent KinectCheckBoxClick;
public event FlatRedBall.Gui.WindowEvent WiiCheckBoxClick;
public event FlatRedBall.Gui.WindowEvent MoveCheckBoxClick;
public event EventHandler BeforeVisibleSet;
public event EventHandler AfterVisibleSet;
protected bool mVisible = true;
public virtual bool Visible
{
	get
	{
		return mVisible;
	}
	set
	{
		bool hasChanged = value != mVisible;
		if (BeforeVisibleSet != null)
		{
			BeforeVisibleSet(this, null);
		}
		mVisible = value;
		FrameInstance.Visible = mVisible;
		KinectCheckBox.Visible = mVisible;
		WiiCheckBox.Visible = mVisible;
		MoveCheckBox.Visible = mVisible;
		BackButton.Visible = mVisible;
		if (AfterVisibleSet != null)
		{
			AfterVisibleSet(this, null);
		}
	}
}
protected Layer LayerProvidedByContainer = null;

        public Options(string contentManagerName) :
            this(contentManagerName, true)
        {
        }


        public Options(string contentManagerName, bool addToManagers) :
			base()
		{
			// Don't delete this:
            ContentManagerName = contentManagerName;
            InitializeEntity(addToManagers);

		}

		protected virtual void InitializeEntity(bool addToManagers)
		{
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			FrameInstance = new RiverGame.Entities.Frame(ContentManagerName, false);
			FrameInstance.Name = "FrameInstance";
			KinectCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			KinectCheckBox.Name = "KinectCheckBox";
			WiiCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			WiiCheckBox.Name = "WiiCheckBox";
			MoveCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			MoveCheckBox.Name = "MoveCheckBox";
			BackButton = new RiverGame.Entities.Button(ContentManagerName, false);
			BackButton.Name = "BackButton";
			BackButton.Click += OnBackButtonClickTunnel;
			KinectCheckBox.Click += OnKinectCheckBoxClickTunnel;
			WiiCheckBox.Click += OnWiiCheckBoxClickTunnel;
			MoveCheckBox.Click += OnMoveCheckBoxClickTunnel;
			
			PostInitialize();
			if (addToManagers)
			{
				AddToManagers(null);
			}


		}

// Generated AddToManagers
		public virtual void AddToManagers (Layer layerToAddTo)
		{
			LayerProvidedByContainer = layerToAddTo;
			SpriteManager.AddPositionedObject(this);
			AddToManagersBottomUp(layerToAddTo);
			CustomInitialize();
		}

		public virtual void Activity()
		{
			// Generated Activity
			
			FrameInstance.Activity();
			KinectCheckBox.Activity();
			WiiCheckBox.Activity();
			MoveCheckBox.Activity();
			BackButton.Activity();
			CustomActivity();
			
			// After Custom Activity
		}

		public virtual void Destroy()
		{
			// Generated Destroy
			SpriteManager.RemovePositionedObject(this);
			if (FrameInstance != null)
			{
				FrameInstance.Destroy();
			}
			if (KinectCheckBox != null)
			{
				KinectCheckBox.Destroy();
			}
			if (WiiCheckBox != null)
			{
				WiiCheckBox.Destroy();
			}
			if (MoveCheckBox != null)
			{
				MoveCheckBox.Destroy();
			}
			if (BackButton != null)
			{
				BackButton.Destroy();
			}
			


			CustomDestroy();
		}

		// Generated Methods
public virtual void PostInitialize ()
{
	Visible = true;
	FrameInstance.ScaleX = 191f;
	FrameInstance.ScaleY = 200f;
	KinectCheckBox.Y = 80f;
	KinectCheckBox.Z = 1f;
	KinectCheckBox.DisplayText = "Kinect";
	WiiCheckBox.Y = 40f;
	WiiCheckBox.Z = 1f;
	WiiCheckBox.DisplayText = "Wii";
	MoveCheckBox.Y = 0f;
	MoveCheckBox.Z = 1f;
	MoveCheckBox.DisplayText = "Move";
	BackButton.DisplayText = "Back";
	BackButton.Y = -80f;
	BackButton.Z = 1f;
}
public virtual void AddToManagersBottomUp (Layer layerToAddTo)
{
	// We move this back to the origin and unrotate it so that anything attached to it can just use its absolute position
	float oldRotationX = RotationX;
	float oldRotationY = RotationY;
	float oldRotationZ = RotationZ;
	
	float oldX = X;
	float oldY = Y;
	float oldZ = Z;
	
	X = 0;
	Y = 0;
	Z = 0;
	RotationX = 0;
	RotationY = 0;
	RotationZ = 0;
	FrameInstance.AddToManagers(layerToAddTo);
	FrameInstance.ScaleX = 191f;
	FrameInstance.ScaleY = 200f;
	if (FrameInstance.Parent == null)
	{
		FrameInstance.AttachTo(this, true);
	}
	KinectCheckBox.AddToManagers(layerToAddTo);
	KinectCheckBox.Y = 80f;
	KinectCheckBox.Z = 1f;
	KinectCheckBox.DisplayText = "Kinect";
	if (KinectCheckBox.Parent == null)
	{
		KinectCheckBox.AttachTo(this, true);
	}
	WiiCheckBox.AddToManagers(layerToAddTo);
	WiiCheckBox.Y = 40f;
	WiiCheckBox.Z = 1f;
	WiiCheckBox.DisplayText = "Wii";
	if (WiiCheckBox.Parent == null)
	{
		WiiCheckBox.AttachTo(this, true);
	}
	MoveCheckBox.AddToManagers(layerToAddTo);
	MoveCheckBox.Y = 0f;
	MoveCheckBox.Z = 1f;
	MoveCheckBox.DisplayText = "Move";
	if (MoveCheckBox.Parent == null)
	{
		MoveCheckBox.AttachTo(this, true);
	}
	BackButton.AddToManagers(layerToAddTo);
	BackButton.DisplayText = "Back";
	BackButton.Y = -80f;
	BackButton.Z = 1f;
	if (BackButton.Parent == null)
	{
		BackButton.AttachTo(this, true);
	}
	X = oldX;
	Y = oldY;
	Z = oldZ;
	RotationX = oldRotationX;
	RotationY = oldRotationY;
	RotationZ = oldRotationZ;
}
public virtual void ConvertToManuallyUpdated ()
{
	this.ForceUpdateDependenciesDeep();
	SpriteManager.ConvertToManuallyUpdated(this);
	FrameInstance.ConvertToManuallyUpdated();
	KinectCheckBox.ConvertToManuallyUpdated();
	WiiCheckBox.ConvertToManuallyUpdated();
	MoveCheckBox.ConvertToManuallyUpdated();
	BackButton.ConvertToManuallyUpdated();
}
public static void LoadStaticContent (string contentManagerName)
{
	ContentManagerName = contentManagerName;
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	if (IsStaticContentLoaded == false)
	{
		IsStaticContentLoaded = true;
		lock (mLockObject)
		{
			if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
			{
				FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("OptionsStaticUnload", UnloadStaticContent);
				mHasRegisteredUnload = true;
			}
		}
		bool registerUnload = false;
		RiverGame.Entities.Frame.LoadStaticContent(contentManagerName);
		RiverGame.Entities.CheckBox.LoadStaticContent(contentManagerName);
		RiverGame.Entities.Button.LoadStaticContent(contentManagerName);
		if (registerUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
		{
			lock (mLockObject)
			{
				if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
				{
					FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("OptionsStaticUnload", UnloadStaticContent);
					mHasRegisteredUnload = true;
				}
			}
		}
		CustomLoadStaticContent(contentManagerName);
	}
}
public static void UnloadStaticContent ()
{
	IsStaticContentLoaded = false;
	mHasRegisteredUnload = false;
}
object GetMember (string memberName)
{
	return null;
}

    }
	
	
	// Extra classes
	public static class OptionsExtensionMethods
	{
		public static void SetVisible (this PositionedObjectList<Options> list, bool value)
		{
			int count = list.Count;
			for (int i = 0; i < count; i++)
			{
				list[i].Visible = value;
			}
		}
	}
	
}
