using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Graphics.Model;

using FlatRedBall.Input;
using FlatRedBall.Utilities;

using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;
using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;
// Generated Usings
using RiverGame.Screens;
using Matrix = Microsoft.Xna.Framework.Matrix;
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;
using FlatRedBall.Math;
using FlatRedBall.Gui;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
#endif

#if FRB_XNA
using Model = Microsoft.Xna.Framework.Graphics.Model;
#endif

namespace RiverGame.Entities
{
	public partial class TextArial : PositionedObject, IDestroyable, IClickable, IVisible
	{
        // This is made global so that static lazy-loaded content can access it.
        public static string ContentManagerName
        {
            get;
            set;
        }

		// Generated Fields
#if DEBUG
static bool HasBeenLoadedWithGlobalContentManager = false;
#endif
static object mLockObject = new object();
static bool mHasRegisteredUnload = false;
static bool IsStaticContentLoaded = false;
private static Scene SceneFile;

private Text TextInstance;
public string DisplayText
{
	get
	{
		return TextInstance.DisplayText;
	}
	set
	{
		TextInstance.DisplayText = value;
	}
}
public HorizontalAlignment HorizontalAlignment
{
	get
	{
		return TextInstance.HorizontalAlignment;
	}
	set
	{
		TextInstance.HorizontalAlignment = value;
	}
}
public event EventHandler BeforeVisibleSet;
public event EventHandler AfterVisibleSet;
protected bool mVisible = true;
public virtual bool Visible
{
	get
	{
		return mVisible;
	}
	set
	{
		bool hasChanged = value != mVisible;
		if (BeforeVisibleSet != null)
		{
			BeforeVisibleSet(this, null);
		}
		mVisible = value;
		TextInstance.Visible = mVisible;
		if (AfterVisibleSet != null)
		{
			AfterVisibleSet(this, null);
		}
	}
}
protected bool mIsPaused;
public override void Pause (InstructionList instructions)
{
	base.Pause(instructions);
	mIsPaused = true;
}
public virtual bool HasCursorOver (Cursor cursor)
{
	if (mIsPaused)
	{
		return false;
	}
	if (!mVisible)
	{
		return false;
	}
	if (LayerProvidedByContainer != null && LayerProvidedByContainer.Visible == false)
	{
		return false;
	}
	if (cursor.IsOn3D(TextInstance, LayerProvidedByContainer))
	{
		return true;
	}
	return false;
}
public virtual bool WasClickedThisFrame (Cursor cursor)
{
	return cursor.PrimaryClick && HasCursorOver(cursor);
}
protected Layer LayerProvidedByContainer = null;

        public TextArial(string contentManagerName) :
            this(contentManagerName, true)
        {
        }


        public TextArial(string contentManagerName, bool addToManagers) :
			base()
		{
			// Don't delete this:
            ContentManagerName = contentManagerName;
            InitializeEntity(addToManagers);

		}

		protected virtual void InitializeEntity(bool addToManagers)
		{
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			TextInstance = SceneFile.Texts.FindByName("Text").Clone();
			TextInstance.AdjustPositionForPixelPerfectDrawing = true;
			
			PostInitialize();
			if (addToManagers)
			{
				AddToManagers(null);
			}


		}

// Generated AddToManagers
		public virtual void AddToManagers (Layer layerToAddTo)
		{
			LayerProvidedByContainer = layerToAddTo;
			SpriteManager.AddPositionedObject(this);
			AddToManagersBottomUp(layerToAddTo);
			CustomInitialize();
		}

		public virtual void Activity()
		{
			// Generated Activity
			mIsPaused = false;
			
			CustomActivity();
			
			// After Custom Activity
		}

		public virtual void Destroy()
		{
			// Generated Destroy
			SpriteManager.RemovePositionedObject(this);
			if (TextInstance != null)
			{
				TextManager.RemoveText(TextInstance);
			}
			


			CustomDestroy();
		}

		// Generated Methods
public virtual void PostInitialize ()
{
	DisplayText = "Hello, I am a Text object";
	X = 0f;
	Y = 0f;
	Z = 0f;
	HorizontalAlignment = FlatRedBall.Graphics.HorizontalAlignment.Center;
}
public virtual void AddToManagersBottomUp (Layer layerToAddTo)
{
	// We move this back to the origin and unrotate it so that anything attached to it can just use its absolute position
	float oldRotationX = RotationX;
	float oldRotationY = RotationY;
	float oldRotationZ = RotationZ;
	
	float oldX = X;
	float oldY = Y;
	float oldZ = Z;
	
	X = 0;
	Y = 0;
	Z = 0;
	RotationX = 0;
	RotationY = 0;
	RotationZ = 0;
	TextManager.AddToLayer(TextInstance, layerToAddTo);
	if (TextInstance.Parent == null)
	{
		TextInstance.AttachTo(this, true);
	}
	X = oldX;
	Y = oldY;
	Z = oldZ;
	RotationX = oldRotationX;
	RotationY = oldRotationY;
	RotationZ = oldRotationZ;
}
public virtual void ConvertToManuallyUpdated ()
{
	this.ForceUpdateDependenciesDeep();
	SpriteManager.ConvertToManuallyUpdated(this);
	TextManager.ConvertToManuallyUpdated(TextInstance);
}
public static void LoadStaticContent (string contentManagerName)
{
	ContentManagerName = contentManagerName;
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	if (IsStaticContentLoaded == false)
	{
		IsStaticContentLoaded = true;
		lock (mLockObject)
		{
			if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
			{
				FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("TextArialStaticUnload", UnloadStaticContent);
				mHasRegisteredUnload = true;
			}
		}
		bool registerUnload = false;
		if (!FlatRedBallServices.IsLoaded<Scene>(@"content/entities/textarial/scenefile.scnx", ContentManagerName))
		{
			registerUnload = true;
		}
		SceneFile = FlatRedBallServices.Load<Scene>(@"content/entities/textarial/scenefile.scnx", ContentManagerName);
		if (registerUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
		{
			lock (mLockObject)
			{
				if (!mHasRegisteredUnload && ContentManagerName != FlatRedBallServices.GlobalContentManager)
				{
					FlatRedBallServices.GetContentManagerByName(ContentManagerName).AddUnloadMethod("TextArialStaticUnload", UnloadStaticContent);
					mHasRegisteredUnload = true;
				}
			}
		}
		CustomLoadStaticContent(contentManagerName);
	}
}
public static void UnloadStaticContent ()
{
	IsStaticContentLoaded = false;
	mHasRegisteredUnload = false;
	if (SceneFile != null)
	{
		SceneFile.RemoveFromManagers(ContentManagerName != "Global");
		SceneFile= null;
	}
}
public static object GetStaticMember (string memberName)
{
	switch(memberName)
	{
		case  "SceneFile":
			return SceneFile;
	}
	return null;
}
object GetMember (string memberName)
{
	switch(memberName)
	{
	}
	return null;
}

    }
	
	
	// Extra classes
	public static class TextArialExtensionMethods
	{
		public static void SetVisible (this PositionedObjectList<TextArial> list, bool value)
		{
			int count = list.Count;
			for (int i = 0; i < count; i++)
			{
				list[i].Visible = value;
			}
		}
	}
	
}
