using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.Math.Geometry;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Input;
using FlatRedBall.IO;
using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using FlatRedBall.Utilities;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;

using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
using Microsoft.Xna.Framework.Media;
#endif

// Generated Usings
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;

namespace RiverGame.Screens
{
	public partial class GameScreen : Screen
	{
		// Generated Fields
		#if DEBUG
		static bool HasBeenLoadedWithGlobalContentManager = false;
		#endif
		public enum VariableState
		{
			Uninitialized, //This exists so that the first set call actually does something
			OkCancelVisible, 
			GameScreenVisible
		}
		private Scene GameScreenBackground;
		private Scene GameScreenBackgroundCollision;
		private Scene PlayerScene;
		
		private RiverGame.Entities.OkCancel OkCancelInstance;
		private Layer Layer2D;
		private RiverGame.Entities.Player PlayerInstance;
		private RiverGame.Entities.Score ScoreInstance;
		private Layer HudLayer;
		public bool OkCancelInstanceVisible
		{
			get
			{
				return OkCancelInstance.Visible;
			}
			set
			{
				OkCancelInstance.Visible = value;
			}
		}
		public event FlatRedBall.Gui.WindowEvent OkCancelInstanceOkClick;
		public event FlatRedBall.Gui.WindowEvent OkCancelInstanceCancelClick;

		public GameScreen()
			: base("GameScreen")
		{
		}

        public override void Initialize(bool addToManagers)
        {
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			GameScreenBackground = FlatRedBallServices.Load<Scene>("content/screens/gamescreenbackground.scnx", ContentManagerName);
			GameScreenBackgroundCollision = FlatRedBallServices.Load<Scene>("content/screens/gamescreenbackgroundcollision.scnx", ContentManagerName);
			PlayerScene = FlatRedBallServices.Load<Scene>("content/screens/playerscene.scnx", ContentManagerName);
			OkCancelInstance = new RiverGame.Entities.OkCancel(ContentManagerName, false);
			OkCancelInstance.Name = "OkCancelInstance";
			PlayerInstance = new RiverGame.Entities.Player(ContentManagerName, false);
			PlayerInstance.Name = "PlayerInstance";
			ScoreInstance = new RiverGame.Entities.Score(ContentManagerName, false);
			ScoreInstance.Name = "ScoreInstance";
			OkCancelInstance.OkClick += OnOkCancelInstanceOkClick;
			OkCancelInstance.OkClick += OnOkCancelInstanceOkClickTunnel;
			OkCancelInstance.CancelClick += OnOkCancelInstanceCancelClick;
			OkCancelInstance.CancelClick += OnOkCancelInstanceCancelClickTunnel;
			
			
			PostInitialize();
			base.Initialize(addToManagers);
			if (addToManagers)
			{
				AddToManagers();
			}

        }
        
// Generated AddToManagers
		public override void AddToManagers ()
		{
			Layer2D = SpriteManager.AddLayer();
			Layer2D.UsePixelCoordinates();
			HudLayer = SpriteManager.AddLayer();
			HudLayer.UsePixelCoordinates();
			AddToManagersBottomUp();
			CustomInitialize();
		}


		public override void Activity(bool firstTimeCalled)
		{
			// Generated Activity
			if (!IsPaused)
			{
				
				OkCancelInstance.Activity();
				PlayerInstance.Activity();
				ScoreInstance.Activity();
			}
			else
			{
			}
			base.Activity(firstTimeCalled);
			if (!IsActivityFinished)
			{
				CustomActivity(firstTimeCalled);
			}
			GameScreenBackground.ManageAll();
			GameScreenBackgroundCollision.ManageAll();
			PlayerScene.ManageAll();


				// After Custom Activity
				
            
		}

		public override void Destroy()
		{
			// Generated Destroy
			if (OkCancelInstance != null)
			{
				OkCancelInstance.Destroy();
			}
			if (Layer2D != null)
			{
				SpriteManager.RemoveLayer(Layer2D);
			}
			if (PlayerInstance != null)
			{
				PlayerInstance.Destroy();
			}
			if (ScoreInstance != null)
			{
				ScoreInstance.Destroy();
			}
			if (HudLayer != null)
			{
				SpriteManager.RemoveLayer(HudLayer);
			}
			GameScreenBackground.RemoveFromManagers(ContentManagerName != "Global");
			GameScreenBackgroundCollision.RemoveFromManagers(ContentManagerName != "Global");
			PlayerScene.RemoveFromManagers(ContentManagerName != "Global");
			

			base.Destroy();

			CustomDestroy();

		}

		// Generated Methods
public virtual void PostInitialize ()
{
	OkCancelInstanceVisible = true;
	CurrentState = GameScreen.VariableState.GameScreenVisible;
	OkCancelInstance.MessageLabelDisplayText = "Exit current game?";
}
public virtual void AddToManagersBottomUp ()
{
	GameScreenBackground.AddToManagers(mLayer);
	GameScreenBackgroundCollision.AddToManagers(mLayer);
	PlayerScene.AddToManagers(mLayer);
	OkCancelInstance.AddToManagers(Layer2D);
	OkCancelInstance.MessageLabelDisplayText = "Exit current game?";
	PlayerInstance.AddToManagers(mLayer);
	ScoreInstance.AddToManagers(HudLayer);
}
public virtual void ConvertToManuallyUpdated ()
{
	GameScreenBackground.ConvertToManuallyUpdated();
	GameScreenBackgroundCollision.ConvertToManuallyUpdated();
	PlayerScene.ConvertToManuallyUpdated();
	OkCancelInstance.ConvertToManuallyUpdated();
	PlayerInstance.ConvertToManuallyUpdated();
	ScoreInstance.ConvertToManuallyUpdated();
}
public static void LoadStaticContent (string contentManagerName)
{
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	RiverGame.Entities.OkCancel.LoadStaticContent(contentManagerName);
	RiverGame.Entities.Player.LoadStaticContent(contentManagerName);
	RiverGame.Entities.Score.LoadStaticContent(contentManagerName);
	CustomLoadStaticContent(contentManagerName);
}
static VariableState mLoadingState = VariableState.Uninitialized;
public static VariableState LoadingState
{
	get
	{
		return mLoadingState;
	}
	set
	{
		mLoadingState = value;
	}
}
VariableState mCurrentState = VariableState.Uninitialized;
public VariableState CurrentState
{
	get
	{
		return mCurrentState;
	}
	set
	{
		mCurrentState = value;
		switch(mCurrentState)
		{
			case  VariableState.OkCancelVisible:
				OkCancelInstanceVisible = true;
				break;
			case  VariableState.GameScreenVisible:
				OkCancelInstanceVisible = false;
				break;
		}
	}
}
public MethodInstruction<GameScreen> InterpolateToState (VariableState stateToInterpolateTo, double secondsToTake)
{
	switch(stateToInterpolateTo)
	{
		case  VariableState.OkCancelVisible:
			break;
		case  VariableState.GameScreenVisible:
			break;
	}
	var instruction = new MethodInstruction<GameScreen>(this, "StopStateInterpolation", new object[]{stateToInterpolateTo}, TimeManager.CurrentTime + secondsToTake);
	InstructionManager.Add(instruction);
	return instruction;
}
public void StopStateInterpolation (VariableState stateToStop)
{
	switch(stateToStop)
	{
		case  VariableState.OkCancelVisible:
			break;
		case  VariableState.GameScreenVisible:
			break;
	}
	CurrentState = stateToStop;
}
public void InterpolateBetween (VariableState firstState, VariableState secondState, float interpolationValue)
{
	switch(firstState)
	{
		case  VariableState.OkCancelVisible:
			break;
		case  VariableState.GameScreenVisible:
			break;
	}
	switch(secondState)
	{
		case  VariableState.OkCancelVisible:
			break;
		case  VariableState.GameScreenVisible:
			break;
	}
}
public override void MoveToState (int state)
{
	this.CurrentState = (VariableState)state;
}

/// <summary>Sets the current state, and pushes that state onto the back stack.</summary>
public void PushState (VariableState state)
{
	this.CurrentState = state;
	
	ScreenManager.PushStateToStack((int)this.CurrentState);
}
object GetMember (string memberName)
{
	switch(memberName)
	{
	}
	return null;
}


	}
}
