using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall;
using FlatRedBall.Input;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Graphics.Animation;
using FlatRedBall.Graphics.Particle;

using FlatRedBall.Graphics.Model;
using FlatRedBall.Math.Geometry;
using FlatRedBall.Math.Splines;

using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;
using FlatRedBall.Localization;

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
using FlatRedBall.Math;
using FlatRedBall.Content.Polygon;
using FlatRedBall.Instructions;
using System.Diagnostics;
using Microsoft.Research.Kinect.Nui;
using System.Threading;
#endif

namespace RiverGame.Screens
{
    public struct PlayerState
    {
        internal Boolean LeaningLeft, LeaningRight, HandLeftUp, HandLeftDown, HandRightUp, HandRightDown, Crouching;
        internal bool StopMoving;

        public void TakeUpperLeft()
        {
            HandLeftUp = true;
            HandLeftDown = false;
            Crouching = false;
        }
        public void TakeUpperRight()
        {
            Crouching = false;
            HandRightUp = true;
            HandRightDown = false;
        }
        public void TakeLowerLeft()
        {
            Crouching = true;
            HandLeftDown = true;
            HandLeftUp = false;
        }
        public void TakeLowerRight()
        {
            Crouching = true;
            HandRightUp = false;
            HandRightDown = true;
        }
        public void Stop()
        {
            LeaningRight = false;
            LeaningLeft = false;
        }
        public void LeanLeft()
        {
            LeaningRight = false;
            LeaningLeft = true;
        }
        public void LeanRight()
        {
            LeaningRight = true;
            LeaningLeft = false;
        }
    }
    public partial class GameScreen
    {

        private Scene CollidableBackgroundSprites;
        private PositionedObjectList<Polygon> CollisionPolygons;
        private PositionedObjectList<Polygon> OldCollisions;
        private int mScoreForTeam0 = 0;

        //we use these variables to keep track if successfull initializations
        private bool InitKinect = false;
        private bool InitWii = false;
        private bool InitMove = false;

        //referances to teh instances of the controllers
        private Kinect kinect;
        private Wii wii;
        private Move move;
        private Thread moveThread;

        //need a framecounter so we don't spam the console too much
        private int FrameCounter;

        //our highest abstraction of the player state
        private PlayerState PlayerState;

        private bool updateMoveThreadQuit;
        public Thread updateMoveThread;
        public static uint update_delay;

        //private Scene Collision;
        //private ShapeCollection GameScreen;
        //Various other variables
        private Screen options;
        private Screen LoadedPopup;
        private float Y_BeforeCollision;

        private void updateMove()
        {
            while (updateMoveThreadQuit == false)
            {


                try
                {

                    Thread.Sleep(Convert.ToInt32(update_delay));
                }
                catch (Exception e)
                {
                    return;
                }
            }

        }
        void CustomInitialize()
        {

            FrameCounter = 0;

            update_delay = 16;
            updateMoveThreadQuit = false;
            Console.WriteLine("Initial X:" + SpriteManager.Camera.X);
            Console.WriteLine("Initial Y:" + SpriteManager.Camera.Y);
            Console.WriteLine("Initial Z:" + SpriteManager.Camera.Z);
            Y_BeforeCollision = PlayerInstance.Y;
            //adjust our view somewhat
            OldCollisions = new PositionedObjectList<Polygon>();
            SpriteManager.Camera.Z = 25;
            PlayerInstance.Z = 2;
            PlayerState.Crouching = false;
            PlayerState.HandLeftDown = false;
            PlayerState.HandLeftUp = false;
            PlayerState.HandRightDown = false;
            PlayerState.HandRightUp = false;
            PlayerState.LeaningLeft = false;
            PlayerState.LeaningRight = false;

            BuildAndPlacePolygons();

            //initialize our devices
            kinect = new Kinect();
            wii = new Wii();
            move = new Move();
            //moveThread = new Thread(new ThreadStart(move.Initialize));
            InitKinect = false;
            InitWii = false;
            InitMove = false;

        }

        //For this prototype we have made a "quick and dirty" playermap in the FlatRedBall GUI editor which we load up and generate some data from
        public void BuildAndPlacePolygons()
        {
            //background sprites
            CollidableBackgroundSprites = GameScreenBackgroundCollision;

            //List for polygons which we need to attack to our sprites to make proper collision detection
            CollisionPolygons = new PositionedObjectList<Polygon>();

            //Create polygons to attach to our sprites for collision detecting (using the entity pattern is recomended from FlatRedBall)
            foreach (Sprite s in CollidableBackgroundSprites.Sprites)
            {
                Polygon p = Polygon.CreateRectangle(s.ScaleX, s.ScaleY);

                p.AttachTo(s, false);
                //polygons named "grass" have a special role as border

                //need to specify different Z values on some sprites to get proper depth
                if (!s.Name.StartsWith("water"))
                {
                    s.Z = 1;
                    if (s.Name.StartsWith("chest") || s.Name.StartsWith("tree") || s.Name.StartsWith("heart"))
                    {
                        s.Z = 2;
                    }
                }

                else { s.Z = 0; }

                //stick to the same names
                p.Name = s.Name;

                //add it to our list which we manipulate in the collisions
                CollisionPolygons.Add(p);

            }
        }

        void CustomActivity(bool firstTimeCalled)
        {

            //Console.WriteLine("LeaningLeft:" + PlayerState.LeaningLeft);
            //Console.WriteLine("LeaningRight:" + PlayerState.LeaningRight);
            FrameCounter++;
            //FlatRedBall.Camera camera = SpriteManager.Camera;

            //Check for pause action
            if (!IsPaused && !PlayerState.StopMoving)
            {
                SpriteManager.Camera.Y += (float)0.01;
                PlayerInstance.Y += (float)0.01;
                Y_BeforeCollision = PlayerInstance.Y;
            }

            //float leftEdge = camera.X - camera.RelativeXEdgeAt(1);

            //FlatRedBall.Camera camera = SpriteManager.Camera;
            //float zPosition = 0; // Default Z value for all placed objects
            //float leftEdge = camera.X - camera.RelativeXEdgeAt(zPosition);
            //float rightEdge = camera.X + camera.RelativeXEdgeAt(zPosition);
            //float bottomEdge = camera.Y - camera.RelativeYEdgeAt(zPosition);
            //float topEdge = camera.Y + camera.RelativeYEdgeAt(zPosition);

            bool Collided = false;
            bool ToTheLeft = false;
            bool ToTheRight = false;

            foreach (Polygon p in CollisionPolygons)
            {
                //we do not collide against water since "water" is the background
                //comparing by names is sub-optimal but works for small amount of objects, the names were defined on sprites when creating the map in the GUI editor
                if ((!((Sprite)p.Parent).Name.StartsWith("water")))
                {
                    Collided = PlayerInstance.Body.CollideAgainstMove(p, 0, 1);
                }

                //bool collided = PlayerInstance.Body.CollideAgainst(p);

                //sequence to check if the player collided against an interactable object
                //interactable objects are in the polygons list, everything else is static scenery
                //after a collision we want to register a change in score and then remove the polygon from the collision list
                //to avoid getting the player stuck since the player and the screen are supposed to move synchronousely
                //when using kinect and wii it's easy to get tangled into objects therefore we just remove objects collided against from the list
                if (Collided)
                {
                    //want to know if the player is to the left or right of the collided object since we need to know that to determine if the right hand was acted with
                    if (PlayerInstance.Body.X > p.X + p.BoundingRadius)
                    {
                        ToTheRight = true;
                        //to the right side
                        //Console.WriteLine("Right side");
                    }
                    else if (PlayerInstance.Body.X < p.X - p.BoundingRadius)
                    {
                        ToTheLeft = true;
                        //to the left side
                        //Console.WriteLine("Left side");
                    }
                    else
                    {
                        ToTheRight = false;
                        ToTheLeft = false;
                    }
                    
                    if (p.Name.StartsWith("rock") && !OldCollisions.Contains(p))
                    {
                        OldCollisions.Add(p);
                        AssignPoint(-1);
                        //polygons.Remove(p);
                    }
                    if (p.Name.StartsWith("tree") && !OldCollisions.Contains(p))
                    {
                        AssignPoint(-1);
                        OldCollisions.Add(p);
                        //polygons.Remove(p);
                    }
                    if (p.Name.StartsWith("dirt") && !OldCollisions.Contains(p))
                    {
                        AssignPoint(-1);
                        OldCollisions.Add(p);
                        //polygons.Remove(p);
                    }
                    if (p.Name.StartsWith("star"))
                    {
                        if (ToTheLeft)
                        {
                            if (PlayerState.HandRightUp)
                            {
                                AssignPoint(2);

                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }
                        }
                        else if (ToTheRight)
                        {
                            if (PlayerState.HandLeftUp)
                            {
                                AssignPoint(2);

                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }
                        }



                    }
                    if (p.Name.StartsWith("chest"))
                    {
                        if (ToTheLeft)
                        {
                            if ((PlayerState.Crouching && PlayerState.HandRightDown))
                            {
                                AssignPoint(2);
                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }
                        }
                        else if (ToTheRight)
                        {
                            if ((PlayerState.Crouching && PlayerState.HandLeftDown))
                            {
                                AssignPoint(2);
                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }

                        }



                    }
                    if (p.Name.StartsWith("heart"))
                    {
                        if (ToTheLeft)
                        {
                            if ((PlayerState.HandRightUp))
                            {
                                AssignPoint(5);
                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }
                        }
                        else if (ToTheRight)
                        {
                            if ((PlayerState.HandLeftUp))
                            {
                                AssignPoint(5);
                                SpriteManager.RemoveSprite((Sprite)p.Parent);
                                CollisionPolygons.Remove(p);
                            }
                        }




                    }
                }

            }

            //need this due to sprite elements some times makes the player get stuck on some non optimal placed sprites and microgaps between sprites
            //also we can not allow the player to stop while the screen is moving up
            PlayerInstance.Y = Y_BeforeCollision;

            if (InputManager.Keyboard.KeyPushed(Keys.Escape))
            {
                if (LoadedPopup == null)
                {

                    this.PauseThisScreen();

                    bool loadNewLayer = false; // or true if you want it on a new layer
                    LoadedPopup = LoadPopup(typeof(MainMenu).FullName, loadNewLayer);

                }
                else if (LoadedPopup != null)
                {
                    this.UnpauseThisScreen();
                    LoadedPopup.Destroy();
                    LoadedPopup = null;


                }


            }

            KeyBoardMove();
            KinectMove();
            WiiMove();
            MoveMove();

            //methods to give visual feedback on the current state of the player as reported by the active input device
            if (PlayerState.LeaningRight)
            {
                PlayerInstance.RotationZ = 12;
            }


            else if (PlayerState.LeaningLeft)
            {
                PlayerInstance.RotationZ = (float)0.5;
            }
            else PlayerInstance.RotationZ = 0;


            if (PlayerState.Crouching)
            {

                PlayerInstance.Sprite.ScaleY = (float)0.5;
            }
            else if (PlayerState.HandLeftUp || PlayerState.HandRightUp)
            {
                PlayerInstance.Sprite.ScaleY = (float)1.5;
            }
            else PlayerInstance.Sprite.ScaleY = 1;


            //want to check for connection all the time to support plugging in/out of devices, the game loop updates 30 times per second
            //so we check about each fourth second

            if (FrameCounter == 90)
            {
                FrameCounter = 0;
                //Console.WriteLine("UseKinect:" + RiverGame.GlobalData.UseKinect);
                if (!InitKinect && GlobalData.UseKinect)
                {

                    InitKinect = kinect.Initialize();


                }
                if (!InitWii && GlobalData.UseWii)
                {

                    InitWii = wii.Initialize();

                }
                if (!InitMove && GlobalData.UseMove)
                {

                    //moveThread.Start();
                    //// Spin for a while waiting for the started thread to become
                    //// alive:
                    //while (!moveThread.IsAlive) ;
                    move.Initialize();
                    InitMove = move.init;
                }
                //Console.WriteLine("HandLeftDown:" + PlayerState.HandLeftDown);
                //Console.WriteLine("HandLeftUp:" + PlayerState.HandLeftUp);
                //Console.WriteLine("HandRightDown:" + PlayerState.HandRightDown);
                //Console.WriteLine("HandRightUp:" + PlayerState.HandRightUp);
            }
            //Need to check if the checkboxes have been unchecked, if so uninitialize the perhepial
            if (!GlobalData.UseKinect && InitKinect)
            {
                kinect.Uninitialize();
            }
            if (InitWii && !GlobalData.UseWii)
            {
                wii.Uninitialize();
            }
            if (InitMove && !GlobalData.UseMove)
            {
                //moveThread.Abort() ;
                move.Uninitialize();
            }
            //Console.WriteLine("LeaningLeft:" + PlayerState.LeaningLeft);
            //Console.WriteLine("LeaningRight:" + PlayerState.LeaningRight);

            ActuateCurrentState();
        }

        private void ActuateCurrentState()
        {
            if (PlayerState.LeaningLeft)
            {
                PlayerInstance.XVelocity = -5;
            }
            else if (PlayerState.LeaningRight)
            {
                PlayerInstance.XVelocity = 5;
            }
            else PlayerInstance.XVelocity = 0;



        }



        //move version which modifies velocity with keyboard
        private void KeyBoardMove()
        {
            if (InputManager.Keyboard.KeyDown(Keys.Up))
            {

                PlayerState.HandLeftUp = true;
                PlayerState.HandRightUp = true;
            }


            else if (InputManager.Keyboard.KeyDown(Keys.Down))
            {
                PlayerState.Crouching = true;
                PlayerState.HandLeftDown = true;
                PlayerState.HandRightDown = true;
            }
            else
            {
                PlayerState.Crouching = false;
                PlayerState.HandLeftUp = false;
                PlayerState.HandRightUp = false;
                PlayerState.HandLeftDown = false;
                PlayerState.HandRightDown = false;
            }


            if (InputManager.Keyboard.KeyDown(Keys.Right))
            {
                PlayerState.LeanRight();

            }
            else if (InputManager.Keyboard.KeyDown(Keys.Left))
            {
                PlayerState.LeanLeft();

            }
            else
            {
                PlayerState.Stop();
            }
            if (PlayerState.StopMoving && InputManager.Keyboard.KeyDown(Keys.P))
            {
                PlayerState.StopMoving = false;
            }
            else if (!PlayerState.StopMoving && InputManager.Keyboard.KeyDown(Keys.P))
            {
                PlayerState.StopMoving = true;
            }


        }

        public void KinectMove()
        {

            if (InitKinect)
            {

                //adjust left/right acording to the weight distribution on the balance board
                if (kinect.State.LeaningLeft)
                {
                    PlayerState.LeaningLeft = true;
                    //Console.WriteLine("Leaning to the LEFT");
                }
                else PlayerState.LeaningLeft = false;

                if (kinect.State.LeaningRight)
                {
                    PlayerState.LeaningRight = true; ;
                    //Console.WriteLine("Leaning to the RIGHT");
                }
                else PlayerState.LeaningRight = false;

                //check the left hand
                if (kinect.State.LeftHandUp)
                {
                    PlayerState.HandLeftUp = true;
                }
                else PlayerState.HandLeftUp = false;
                if (kinect.State.LeftHandDown)
                {
                    PlayerState.HandLeftDown = true;
                }
                else PlayerState.HandLeftDown = false;

                //check the right hand
                if (kinect.State.RightHandUp)
                {
                    PlayerState.HandRightUp = true;
                }
                else PlayerState.HandRightUp = false;

                if (kinect.State.RightHandDown)
                {
                    PlayerState.HandRightDown = true;
                }
                else PlayerState.HandRightDown = false;

                if (kinect.State.BentDown)
                {

                    //PlayerState.HandLeftDown = true;
                    PlayerState.Crouching = true;

                }
                else
                {
                    //PlayerState.HandLeftDown = false;
                    PlayerState.Crouching = false;

                }

                //if (kinect.State.BentDown && kinect.State.RightHandDown)
                //{

                //    PlayerState.HandRightDown = true;
                //    PlayerState.Crouching = true;

                //}
                //else
                //{
                //    PlayerState.HandRightDown = false;
                //    PlayerState.Crouching = false;

                //}
            }
            //Console.WriteLine("Playerstate.LeftHandUp"+PlayerState.HandLeftUp);
            //Console.WriteLine("kinect.LeftHandUp" + kinect.State.LeftHandUp);
        }

        public void WiiMove()
        {

            if (InitWii)
            {
                //adjust left/right acording to the weight distribution on the balance board
                if (wii.State.LeaningLeft)
                {
                    PlayerState.LeaningLeft = true;

                }
                else PlayerState.LeaningLeft = false;


                if (wii.State.LeaningRight)
                {
                    PlayerState.LeaningRight = true;


                }
                else PlayerState.LeaningRight = false;



                if (wii.State.LeftHandAction)
                {

                    PlayerState.Crouching = true;
                    PlayerState.HandLeftDown = true;
                    PlayerState.HandLeftUp = true;

                }
                else
                {
                    PlayerState.Crouching = false;
                    PlayerState.HandLeftDown = false;
                    PlayerState.HandLeftUp = false;

                }

                if (wii.State.RightHandAction)
                {
                    PlayerState.Crouching = true;
                    PlayerState.HandRightUp = true;
                    PlayerState.HandRightDown = true;

                }
                else
                {
                    PlayerState.Crouching = false;
                    PlayerState.HandRightUp = false;
                    PlayerState.HandRightDown = false;
                }
                //use two wii motes, try with "best effort" to determine if a hand is raised or lowered
                //difficult to determine if the player actually bends down to pick up as intented

            }
        }

        public void MoveMove()
        {

            if (InitMove)
            {
                move.updateState();
                MoveState state = move.MoveState;

                if (state.StopMoving)
                {
                    PlayerState.StopMoving = true;
                }
                if (state.LeaningLeft)
                {
                    PlayerState.LeaningLeft = true;

                }
                else PlayerState.LeaningLeft = false;


                if (state.LeaningRight)
                {
                    PlayerState.LeaningRight = true;

                }
                else PlayerState.LeaningRight = false;


                //checking for left hand states
                if (state.LeftHandUp)
                {

                    //LeftHandDownAndBent = true;
                    PlayerState.HandLeftUp = true;


                }
                else
                {
                    //LeftHandDownAndBent = false;
                    PlayerState.HandLeftUp = false;

                }
                if (state.LeftHandDown)
                {
                    PlayerState.HandLeftDown = true;
                }
                else PlayerState.HandLeftDown = false;

                //checking for right hand states
                if (state.RightHandUp)
                {

                    //RightHandDownAndBent = true;
                    PlayerState.HandRightUp = true;

                }
                else
                {
                    //RightHandDownAndBent = false;
                    PlayerState.HandRightUp = false;

                }

                if (state.RightHandDown)
                {
                    PlayerState.HandRightDown = true;

                }
                else PlayerState.HandRightDown = false;

                if (state.BentDown)
                {
                    PlayerState.Crouching = true;


                }
                else PlayerState.Crouching = false;
            }

        }


        void AssignPoint(int amount)
        {
            mScoreForTeam0 += amount;
            ScoreInstance.Score1 = mScoreForTeam0;
        }

        void CustomDestroy()
        {
            //Example of cleaning up
            //GameScreen.RemoveFromManagers(ContentManagerName != "Global");
            if (InitKinect)
            {
                kinect.Uninitialize();
            }
            if (InitMove)
            {
                move.Uninitialize();

            }
        }

        static void CustomLoadStaticContent(string contentManagerName)
        {


        }

    }
}
