﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Research.Kinect.Nui;
using System.Linq;
using Coding4Fun.Kinect.Wpf;

namespace RiverGame.Screens
{
    public class Kinect
    {
        private Runtime nui = new Runtime();
        private Joint RightHand;
        private Joint LeftHand;
        private Joint Spine;
        private Joint Hip;
        private Joint ShoulderCenter;
        private Joint Knee;
        private KinectState state;

        public struct KinectState
        {
            public Boolean LeaningLeft, LeaningRight, LeftHandUp, LeftHandDown, RightHandUp, RightHandDown, BentDown;


        }
        public KinectState State
        {
            get { return state; }
            set { state = value; }
        }

        public Kinect()
        {


        }

        //initialize the kinect framework
        //returns boolean with true if it worked, false if something went wrong
        public Boolean Initialize()
        {

            Boolean init = false;

            try
            {
                nui.Initialize(RuntimeOptions.UseSkeletalTracking);
                //Must set to true and set after call to Initialize 
                nui.SkeletonEngine.TransformSmooth = true;

                //Use to transform and reduce jitter 
                var parameters = new TransformSmoothParameters
                {
                    Smoothing = 0.75f,
                    Correction = 0.0f,
                    Prediction = 0.0f,
                    JitterRadius = 0.05f,
                    MaxDeviationRadius = 0.04f
                };

                nui.SkeletonEngine.SmoothParameters = parameters;


                nui.SkeletonFrameReady += new EventHandler<SkeletonFrameReadyEventArgs>(nui_SkeletonFrameReady);
                init = true;
            }
            catch (Exception e)
            {
                Console.WriteLine("Kinect init failed");
            }
            return init;
        }

        //Event handler for receiving skeleton events
        void nui_SkeletonFrameReady(object sender, SkeletonFrameReadyEventArgs e)
        {

            SkeletonFrame skeletonFrame = e.SkeletonFrame;
            //SkeletonFrame skeletonFrame = e.SkeletonFrame;

            foreach (SkeletonData data in skeletonFrame.Skeletons)
            {
                if (SkeletonTrackingState.Tracked == data.TrackingState)
                {

                }
            }

            //get the first tracked skeleton 
            SkeletonData skeleton = (from s in skeletonFrame.Skeletons
                                     where s.TrackingState == SkeletonTrackingState.Tracked
                                     select s).FirstOrDefault();

            //need to check that we got a skeleton lock before trying to process it
            if (skeleton != null)
            {
                // the various joints  
                RightHand = 
                    skeleton.Joints[JointID.HandRight].ScaleTo(1024, 720, .5f, .5f);
                LeftHand = 
                    skeleton.Joints[JointID.HandLeft].ScaleTo(1024, 720, .5f, .5f);
                Spine = 
                    skeleton.Joints[JointID.Spine].ScaleTo(1024, 720, .5f, .5f);
                Hip = 
                    skeleton.Joints[JointID.HipCenter].ScaleTo(1024, 720, .5f, .5f);
                ShoulderCenter = 
                    skeleton.Joints[JointID.ShoulderCenter].ScaleTo(1024, 720, .5f, .5f);
                                
                //now need to check if the shoulder is to the left or right in relation to the hip
                if (ShoulderCenter.Position.X > (Hip.Position.X + 50))
                {
                    //leaning to the right
                    state.LeaningRight = true;
                    
                }
                else state.LeaningRight = false;

                if (ShoulderCenter.Position.X < (Hip.Position.X - 50))
                {
                    //leaning to the left
                    state.LeaningLeft = true;      
                }
                else state.LeaningLeft = false;                  

                if (RightHand.Position.Y <50)
                {
                    //right hand is lifted over the head, 
                    //could also check for distance from the spine/shoulder in X direction
                    state.RightHandUp = true;
                    state.RightHandDown = false;
       
                }
                else if (RightHand.Position.Y > 250)
                {
                    state.RightHandUp = false;
                    state.RightHandDown = true;
      
                }
                
                if (LeftHand.Position.Y < 50)
                {
                    //left hand is lifted over the head, 
                    //could also check for distance from the spine/shoulder in X direction
                    state.LeftHandUp = true;
                    state.LeftHandDown = false;
                }
                else if (LeftHand.Position.Y > 250)
                {
                    state.LeftHandUp = false;
                    state.LeftHandDown = true;
                }

                if (ShoulderCenter.Position.Y>250)
                {
                    state.BentDown = true;
                    //the hip is very close to the knee so we assume a crouching position

                }
                else state.BentDown = false;
            }

        }



        public void Uninitialize()
        {
            nui.Uninitialize();
        }
    }
}
