using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.Math.Geometry;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Input;
using FlatRedBall.IO;
using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using FlatRedBall.Utilities;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;

using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
using Microsoft.Xna.Framework.Media;
#endif

// Generated Usings
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;

namespace RiverGame.Screens
{
	public partial class MainMenu : Screen
	{
		// Generated Fields
		#if DEBUG
		static bool HasBeenLoadedWithGlobalContentManager = false;
		#endif
		public enum VariableState
		{
			Uninitialized, //This exists so that the first set call actually does something
			MainButtonsVisible, 
			ExitOkCancelVisible, 
			OptionsVisible
		}
		
		private RiverGame.Entities.Button StartGameButton;
		private RiverGame.Entities.Button OptionsButton;
		private RiverGame.Entities.Button ExitGame;
		private Layer Layer2D;
		private RiverGame.Entities.OkCancel OkCancelInstance;
		private Layer OkCancelLayer;
		private RiverGame.Entities.Options OptionsInstance;
		private Layer OptionsLayer;
		public bool OkCancelInstanceVisible
		{
			get
			{
				return OkCancelInstance.Visible;
			}
			set
			{
				OkCancelInstance.Visible = value;
			}
		}
		public bool StartGameButtonVisible
		{
			get
			{
				return StartGameButton.Visible;
			}
			set
			{
				StartGameButton.Visible = value;
			}
		}
		public bool ExitGameVisible
		{
			get
			{
				return ExitGame.Visible;
			}
			set
			{
				ExitGame.Visible = value;
			}
		}
		public bool OptionsInstanceVisible
		{
			get
			{
				return OptionsInstance.Visible;
			}
			set
			{
				OptionsInstance.Visible = value;
			}
		}
		public event FlatRedBall.Gui.WindowEvent ExitGameClick;
		public event FlatRedBall.Gui.WindowEvent OptionsButtonClick;
		public event FlatRedBall.Gui.WindowEvent OkCancelInstanceOkClick;
		public event FlatRedBall.Gui.WindowEvent OkCancelInstanceCancelClick;
		public event FlatRedBall.Gui.WindowEvent StartGameButtonClick;
		public event FlatRedBall.Gui.WindowEvent OptionsInstanceBackButtonClick;
		public event FlatRedBall.Gui.WindowEvent OptionsInstanceKinectCheckBoxClick;
		public event FlatRedBall.Gui.WindowEvent OptionsInstanceMoveCheckBoxClick;
		public event FlatRedBall.Gui.WindowEvent OptionsInstanceWiiCheckBoxClick;

		public MainMenu()
			: base("MainMenu")
		{
		}

        public override void Initialize(bool addToManagers)
        {
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			StartGameButton = new RiverGame.Entities.Button(ContentManagerName, false);
			StartGameButton.Name = "StartGameButton";
			OptionsButton = new RiverGame.Entities.Button(ContentManagerName, false);
			OptionsButton.Name = "OptionsButton";
			ExitGame = new RiverGame.Entities.Button(ContentManagerName, false);
			ExitGame.Name = "ExitGame";
			OkCancelInstance = new RiverGame.Entities.OkCancel(ContentManagerName, false);
			OkCancelInstance.Name = "OkCancelInstance";
			OptionsInstance = new RiverGame.Entities.Options(ContentManagerName, false);
			OptionsInstance.Name = "OptionsInstance";
			ExitGame.Click += OnExitGameClick;
			ExitGame.Click += OnExitGameClickTunnel;
			OptionsButton.Click += OnOptionsButtonClick;
			OptionsButton.Click += OnOptionsButtonClickTunnel;
			OkCancelInstance.OkClick += OnOkCancelInstanceOkClick;
			OkCancelInstance.OkClick += OnOkCancelInstanceOkClickTunnel;
			OkCancelInstance.CancelClick += OnOkCancelInstanceCancelClick;
			OkCancelInstance.CancelClick += OnOkCancelInstanceCancelClickTunnel;
			StartGameButton.Click += OnStartGameButtonClick;
			StartGameButton.Click += OnStartGameButtonClickTunnel;
			OptionsInstance.BackButtonClick += OnOptionsInstanceBackButtonClick;
			OptionsInstance.BackButtonClick += OnOptionsInstanceBackButtonClickTunnel;
			OptionsInstance.KinectCheckBoxClick += OnOptionsInstanceKinectCheckBoxClick;
			OptionsInstance.KinectCheckBoxClick += OnOptionsInstanceKinectCheckBoxClickTunnel;
			OptionsInstance.MoveCheckBoxClick += OnOptionsInstanceMoveCheckBoxClick;
			OptionsInstance.MoveCheckBoxClick += OnOptionsInstanceMoveCheckBoxClickTunnel;
			OptionsInstance.WiiCheckBoxClick += OnOptionsInstanceWiiCheckBoxClick;
			OptionsInstance.WiiCheckBoxClick += OnOptionsInstanceWiiCheckBoxClickTunnel;
			
			
			PostInitialize();
			base.Initialize(addToManagers);
			if (addToManagers)
			{
				AddToManagers();
			}

        }
        
// Generated AddToManagers
		public override void AddToManagers ()
		{
			Layer2D = SpriteManager.AddLayer();
			Layer2D.UsePixelCoordinates();
			OkCancelLayer = SpriteManager.AddLayer();
			OkCancelLayer.UsePixelCoordinates();
			OptionsLayer = SpriteManager.AddLayer();
			OptionsLayer.UsePixelCoordinates();
			AddToManagersBottomUp();
			CustomInitialize();
		}


		public override void Activity(bool firstTimeCalled)
		{
			// Generated Activity
			if (!IsPaused)
			{
				
				StartGameButton.Activity();
				OptionsButton.Activity();
				ExitGame.Activity();
				OkCancelInstance.Activity();
				OptionsInstance.Activity();
			}
			else
			{
			}
			base.Activity(firstTimeCalled);
			if (!IsActivityFinished)
			{
				CustomActivity(firstTimeCalled);
			}


				// After Custom Activity
				
            
		}

		public override void Destroy()
		{
			// Generated Destroy
			if (StartGameButton != null)
			{
				StartGameButton.Destroy();
			}
			if (OptionsButton != null)
			{
				OptionsButton.Destroy();
			}
			if (ExitGame != null)
			{
				ExitGame.Destroy();
			}
			if (Layer2D != null)
			{
				SpriteManager.RemoveLayer(Layer2D);
			}
			if (OkCancelInstance != null)
			{
				OkCancelInstance.Destroy();
			}
			if (OkCancelLayer != null)
			{
				SpriteManager.RemoveLayer(OkCancelLayer);
			}
			if (OptionsInstance != null)
			{
				OptionsInstance.Destroy();
			}
			if (OptionsLayer != null)
			{
				SpriteManager.RemoveLayer(OptionsLayer);
			}
			

			base.Destroy();

			CustomDestroy();

		}

		// Generated Methods
public virtual void PostInitialize ()
{
	OkCancelInstanceVisible = true;
	StartGameButtonVisible = true;
	ExitGameVisible = true;
	OptionsInstanceVisible = true;
	CurrentState = MainMenu.VariableState.MainButtonsVisible;
	StartGameButton.DisplayText = "Start";
	OptionsButton.DisplayText = "Options";
	OptionsButton.Y = -40f;
	ExitGame.DisplayText = "Exit Game";
	ExitGame.Y = -80f;
	OkCancelInstance.MessageLabelDisplayText = "Exit game?";
}
public virtual void AddToManagersBottomUp ()
{
	StartGameButton.AddToManagers(Layer2D);
	StartGameButton.DisplayText = "Start";
	OptionsButton.AddToManagers(Layer2D);
	OptionsButton.DisplayText = "Options";
	OptionsButton.Y = -40f;
	ExitGame.AddToManagers(Layer2D);
	ExitGame.DisplayText = "Exit Game";
	ExitGame.Y = -80f;
	OkCancelInstance.AddToManagers(OkCancelLayer);
	OkCancelInstance.MessageLabelDisplayText = "Exit game?";
	OptionsInstance.AddToManagers(OptionsLayer);
}
public virtual void ConvertToManuallyUpdated ()
{
	StartGameButton.ConvertToManuallyUpdated();
	OptionsButton.ConvertToManuallyUpdated();
	ExitGame.ConvertToManuallyUpdated();
	OkCancelInstance.ConvertToManuallyUpdated();
	OptionsInstance.ConvertToManuallyUpdated();
}
public static void LoadStaticContent (string contentManagerName)
{
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	RiverGame.Entities.Button.LoadStaticContent(contentManagerName);
	RiverGame.Entities.OkCancel.LoadStaticContent(contentManagerName);
	RiverGame.Entities.Options.LoadStaticContent(contentManagerName);
	CustomLoadStaticContent(contentManagerName);
}
static VariableState mLoadingState = VariableState.Uninitialized;
public static VariableState LoadingState
{
	get
	{
		return mLoadingState;
	}
	set
	{
		mLoadingState = value;
	}
}
VariableState mCurrentState = VariableState.Uninitialized;
public VariableState CurrentState
{
	get
	{
		return mCurrentState;
	}
	set
	{
		mCurrentState = value;
		switch(mCurrentState)
		{
			case  VariableState.MainButtonsVisible:
				OkCancelInstanceVisible = false;
				StartGameButtonVisible = true;
				ExitGameVisible = true;
				OptionsInstanceVisible = false;
				break;
			case  VariableState.ExitOkCancelVisible:
				OkCancelInstanceVisible = true;
				StartGameButtonVisible = false;
				ExitGameVisible = false;
				OptionsInstanceVisible = false;
				break;
			case  VariableState.OptionsVisible:
				OkCancelInstanceVisible = false;
				StartGameButtonVisible = false;
				ExitGameVisible = false;
				OptionsInstanceVisible = true;
				break;
		}
	}
}
public MethodInstruction<MainMenu> InterpolateToState (VariableState stateToInterpolateTo, double secondsToTake)
{
	switch(stateToInterpolateTo)
	{
		case  VariableState.MainButtonsVisible:
			break;
		case  VariableState.ExitOkCancelVisible:
			break;
		case  VariableState.OptionsVisible:
			break;
	}
	var instruction = new MethodInstruction<MainMenu>(this, "StopStateInterpolation", new object[]{stateToInterpolateTo}, TimeManager.CurrentTime + secondsToTake);
	InstructionManager.Add(instruction);
	return instruction;
}
public void StopStateInterpolation (VariableState stateToStop)
{
	switch(stateToStop)
	{
		case  VariableState.MainButtonsVisible:
			break;
		case  VariableState.ExitOkCancelVisible:
			break;
		case  VariableState.OptionsVisible:
			break;
	}
	CurrentState = stateToStop;
}
public void InterpolateBetween (VariableState firstState, VariableState secondState, float interpolationValue)
{
	switch(firstState)
	{
		case  VariableState.MainButtonsVisible:
			break;
		case  VariableState.ExitOkCancelVisible:
			break;
		case  VariableState.OptionsVisible:
			break;
	}
	switch(secondState)
	{
		case  VariableState.MainButtonsVisible:
			break;
		case  VariableState.ExitOkCancelVisible:
			break;
		case  VariableState.OptionsVisible:
			break;
	}
}
public override void MoveToState (int state)
{
	this.CurrentState = (VariableState)state;
}

/// <summary>Sets the current state, and pushes that state onto the back stack.</summary>
public void PushState (VariableState state)
{
	this.CurrentState = state;
	
	ScreenManager.PushStateToStack((int)this.CurrentState);
}
object GetMember (string memberName)
{
	return null;
}


	}
}
