﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;
using System.Threading;

namespace RiverGame.Screens
{
    public struct MoveState
    {
        public Boolean LeaningLeft, LeaningRight, LeftHandUp, LeftHandDown, RightHandUp, RightHandDown, BentDown;
        public Boolean Initialized;
        public MoveController ControllerLeftHand, ControllerRightHand, ControllerCenter;
        public Boolean StopMoving;

    }


    public struct MoveController
    {
        public PSMoveSharpGemState GemState;
        public PSMoveSharpPointerState PointerState;
        public PSMoveSharpSphereState SphereState;
        public PSMoveSharpPadData PadData;
        public MoveControllerPosition Position;


    }

    public struct MoveControllerPosition
    {

        public float Initial_X, Initial_Y;
        public float Current_X, Current_Y;

    }

    public class Move
    {

        public static PSMoveClientThreadedRead moveClient;
        public static bool client_connected;
        public static bool client_paused;
        public static int selected_move;
        public static int selected_nav;
        public static uint update_delay;
        public static bool image_paused;
        public static uint image_update_delay;
        public static bool sliced_image_paused;
        public static uint sliced_image_update_delay;
        public bool init;
        public static bool[] reset_enabled;
        private MoveControllerPosition LeftHand, RightHand, Center;
        PSMoveSharpGemState GemLeft, GemRight, GemCenter;
        private String adress;
        private int port;
        private MoveState moveState;
        public enum ClientCalibrationStep
        {
            Left = 0,
            Right = 1,
            Bottom = 2,
            Top = 3,
            Done = 4
        }

        public String Adress
        {
            get { return adress; }
            set { adress = value; }

        }

        public MoveState MoveState
        {
            get { return moveState; }
            set { moveState = value; }

        }
        public int Port
        {
            get { return port; }
            set { port = value; }

        }
        static UInt32 processed_packet_index = 0;
        static UInt16[] last_buttons = new UInt16[4];
        static ClientCalibrationStep[] calibration_step = new ClientCalibrationStep[4];

        public Move()
        {

            init = false;
            client_connected = false;
            client_paused = false;
            selected_move = 0;
            selected_nav = 0;
            update_delay = 16;
            reset_enabled = new bool[PSMoveSharpState.PSMoveSharpNumMoveControllers];
            image_paused = true;
            image_update_delay = 32;
            sliced_image_paused = true;
            sliced_image_update_delay = 32;
            for (int i = 0; i < PSMoveSharpState.PSMoveSharpNumMoveControllers; i++)
            {
                reset_enabled[i] = true;
            }
            LeftHand = new MoveControllerPosition();
            RightHand = new MoveControllerPosition();
            Center = new MoveControllerPosition();
        }

        public void updateState()
        {
            //get the latest state info from the server
            PSMoveSharpState state = moveClient.GetLatestState();
            PSMoveSharpCameraFrameState camera_frame_state = moveClient.GetLatestCameraFrameState();
            if (processed_packet_index == state.packet_index)
            {
                return;
            }
            processed_packet_index = state.packet_index;

            //updateTabPageNav(state);
            //labelPositionPos1ValX.Text = (state.positionPointerStates[0].valid == 1) ? state.positionPointerStates[0].normalized_x.ToString("N3") : "N/A";
            moveState.ControllerLeftHand.GemState = state.gemStates[0];
            moveState.ControllerLeftHand.Position.Current_X = moveState.ControllerLeftHand.GemState.pos.x;
            moveState.ControllerLeftHand.Position.Current_Y = moveState.ControllerLeftHand.GemState.pos.y;

            //moveState.ControllerLeftHand.Position
            if (state.gemStates[1].tracking_flags > 0)
            {
                moveState.ControllerRightHand.GemState = state.gemStates[1];
                moveState.ControllerRightHand.Position.Current_X = moveState.ControllerRightHand.GemState.pos.x;
                moveState.ControllerRightHand.Position.Current_Y = moveState.ControllerRightHand.GemState.pos.y;

                if (state.gemStates[2].tracking_flags > 0)
                {
                    moveState.ControllerCenter.GemState = state.gemStates[2];
                    moveState.ControllerCenter.Position.Current_X = moveState.ControllerCenter.GemState.pos.x;
                    moveState.ControllerCenter.Position.Current_Y = moveState.ControllerCenter.GemState.pos.y;
                }

            }


            //for (int i = 0; i < PSMoveSharpState.PSMoveSharpNumMoveControllers; i++)
            //{
            //    UInt16 just_pressed;

            //    {
            //        UInt16 changed_buttons = (UInt16)(state.gemStates[i].pad.digitalbuttons ^ last_buttons[i]);
            //        just_pressed = (UInt16)(changed_buttons & state.gemStates[i].pad.digitalbuttons);
            //        last_buttons[i] = state.gemStates[i].pad.digitalbuttons;
            //        Console.WriteLine("Pressed button:" + just_pressed + " on controller:" + i);
            //    }
            //}moveClient.CalibrateController(0);
            //if (((moveState.ControllerLeftHand.GemState.pad.digitalbuttons & PSMoveSharpConstants.ctrlCircle) != 0))
            //{
            //    Console.WriteLine("Pressed Circle, sending calibration command");
            //    moveClient.CalibrateController(0);
            //    if (state.gemStates.Count() > 1)
            //    {
            //        moveClient.CalibrateController(1);
            //        if (state.gemStates.Count() > 2)
            //        {
            //            moveClient.CalibrateController(2);
            //        }

            //    }
            //}

            //user clicked the triangel on the controller, so we record the initial data
            //Console.WriteLine("-------------------");
            //Console.WriteLine(state.gemStates[0].tracking_flags);
            //Console.WriteLine("right x"+moveState.ControllerRightHand.GemState.pos.x);
            //Console.WriteLine("right y"+moveState.ControllerRightHand.GemState.pos.y);
            //Console.WriteLine(state.gemStates[1].tracking_flags);
            //Console.WriteLine(state.gemStates[2].tracking_flags);
            //Console.WriteLine("Current X,Y" + moveState.ControllerLeftHand.GemState.pos.x + "," + moveState.ControllerLeftHand.GemState.pos.y);
            //Console.WriteLine("Initial X,Y" + moveState.ControllerLeftHand.Position.Initial_X + "," + moveState.ControllerLeftHand.Position.Initial_Y);
            if (((moveState.ControllerLeftHand.GemState.pad.digitalbuttons & PSMoveSharpConstants.ctrlTriangle) != 0))
            {
                //left controller
                moveState.ControllerLeftHand.Position.Initial_X = moveState.ControllerLeftHand.GemState.pos.x;
                moveState.ControllerLeftHand.Position.Initial_Y = moveState.ControllerLeftHand.GemState.pos.y;
                Console.WriteLine("Triangle pushed, calibrated LeftHandto Initial position with x,y:" + moveState.ControllerLeftHand.GemState.pos.x + "," + moveState.ControllerLeftHand.GemState.pos.y);

                if (state.gemStates[1].tracking_flags > 0)
                {
                    Console.WriteLine("calibrated RightHand");

                    //right controller
                    moveState.ControllerRightHand.Position.Initial_X = moveState.ControllerRightHand.GemState.pos.x;
                    moveState.ControllerRightHand.Position.Initial_Y = moveState.ControllerRightHand.GemState.pos.y;
                    //center controller
                 
                }

                if (state.gemStates[2].tracking_flags > 0)
                {
                    Console.WriteLine("calibrated Center");
                    moveState.ControllerCenter.Position.Initial_X = moveState.ControllerCenter.GemState.pos.x;
                    moveState.ControllerCenter.Position.Initial_Y = moveState.ControllerCenter.GemState.pos.y;
                }
                Console.WriteLine("Triangle pushed, calibrated to Initial position with x,y:" + moveState.ControllerLeftHand.GemState.pos.x + "," + moveState.ControllerLeftHand.GemState.pos.y);

            }
            else
            {
                if (moveState.ControllerLeftHand.Position.Current_Y > moveState.ControllerLeftHand.Position.Initial_Y + 200)
                {
                    moveState.LeftHandUp = true;
                    
                    //moveState.LeftHandDown = false;
                }
                else if (moveState.ControllerLeftHand.Position.Current_Y < moveState.ControllerLeftHand.Position.Initial_Y - 200)
                {
                    moveState.LeftHandDown = true;
                    
                    //moveState.LeftHandUp = false;
                }
                else
                {
                    moveState.LeftHandDown = false;
                    moveState.LeftHandUp = false;
                }

                //if (moveState.ControllerLeftHand.Position.Current_X > moveState.ControllerLeftHand.Position.Initial_X + 100)
                //{
                //    moveState.LeaningLeft = false;
                //}
                //else if (moveState.ControllerLeftHand.Position.Current_X < moveState.ControllerLeftHand.Position.Initial_X - 100)
                //{
                //    moveState.LeaningLeft = true;
                //}
                //else
                //{
                //    moveState.LeaningLeft = false;

                //}

                if (state.gemStates[1].tracking_flags > 0)
                {

                    //Right hand
                    if (moveState.ControllerRightHand.Position.Current_Y > moveState.ControllerRightHand.Position.Initial_Y + 200)
                    {
                        moveState.RightHandUp = true;
                        //moveState.LeftHandDown = false;
                    }
                    else if (moveState.ControllerRightHand.Position.Current_Y < moveState.ControllerRightHand.Position.Initial_Y - 200)
                    {
                        moveState.RightHandDown = true;
                        //moveState.RightHandUp = false;
                    }
                    else
                    {
                        moveState.RightHandDown = false;
                        moveState.RightHandUp = false;
                    }

                    //if (moveState.ControllerRightHand.Position.Current_X > moveState.ControllerRightHand.Position.Initial_X + 100)
                    //{
                    //    moveState.LeaningRight = true;
                    //}
                    //else if (moveState.ControllerRightHand.Position.Current_X < moveState.ControllerRightHand.Position.Initial_X - 100)
                    //{
                    //    moveState.LeaningRight = false;
                    //}
                    //else
                    //{

                    //    moveState.LeaningRight = false;
                    //}
                    //center

                    if (state.gemStates[2].tracking_flags > 0)
                    {
                        Console.WriteLine("CenterCurrentY:" + moveState.ControllerCenter.Position.Current_Y);
                        Console.WriteLine("CenterinitialY:" + moveState.ControllerCenter.Position.Initial_Y);

                        Console.WriteLine("CenterCurrentX:" + moveState.ControllerCenter.Position.Current_X);
                        Console.WriteLine("CenterinitialX:" + moveState.ControllerCenter.Position.Initial_X);

                        if (moveState.ControllerCenter.Position.Current_Y < moveState.ControllerCenter.Position.Initial_Y - 100)
                        {
                            moveState.BentDown = true;
                            //moveState.LeftHandDown = false;
                        }
          
                        else
                        {
                            moveState.BentDown = false;

                        }
                        if (moveState.ControllerCenter.Position.Current_X < moveState.ControllerCenter.Position.Initial_Y -100) {
                            moveState.LeaningLeft = true;
                        }
                        else if (moveState.ControllerCenter.Position.Current_X > moveState.ControllerCenter.Position.Initial_Y + 100)
                        {
                            moveState.LeaningRight = true;
                        }
                        else {
                            moveState.LeaningRight = false;
                            moveState.LeaningLeft = false;
                        }
                    }

                }
            }
            if (((moveState.ControllerLeftHand.GemState.pad.digitalbuttons & PSMoveSharpConstants.ctrlCross) != 0)) {
                moveState.StopMoving = true;
            }




            //if (moveState.Initialized == null) {

            //    moveState.Position.Initial_X = (state.positionPointerStates[0].valid == 1) ? state.positionPointerStates[0].normalized_x : 0;
            //    moveState.Position.Initial_Y = (state.positionPointerStates[0].valid == 1) ? state.positionPointerStates[0].normalized_y : 0;
            //}
            //moveState.Position.Current_X = (state.positionPointerStates[0].valid == 1) ? state.positionPointerStates[0].normalized_x : 0;
            //moveState.Position.Current_Y = (state.positionPointerStates[0].valid == 1) ? state.positionPointerStates[0].normalized_y : 0;



        }
        public static void client_connect(String server_address, int server_port)
        {
            moveClient = new PSMoveClientThreadedRead();

            if (!client_connected)
            {
                try
                {
                    moveClient.Connect(Dns.GetHostAddresses(server_address)[0].ToString(), server_port);
                    moveClient.StartThread();
                    client_connected = true;
                }
                catch
                {
                    client_connected = false;
                    return;
                }

               

            }
            //Properties.Settings.Default.most_recent_server = server_address;
            //Properties.Settings.Default.Save();
        }

        public static void client_disconnect()
        {
            try
            {
                client_paused = false;

                moveClient.StopThread();
                moveClient.Close();
            }
            catch
            {
                return;
            }

            client_connected = false;
        }


        public void Initialize()
        {
            if (!client_connected)
            {
                try
                {
                    client_connect(GlobalData.MoveIP, GlobalData.MovePort);
                    init = true;
                    //moveClient.CalibrateController(0);
                    Console.WriteLine("Move connected");
                    
                }
                catch (Exception e)
                {
                    Console.WriteLine("Move connect failed");
                    GlobalData.UseMove = false;
                    init = false;
                }


            }
        }

        public void Uninitialize()
        {

            client_disconnect();
        }
    }
}
