using System;
using System.Collections.Generic;
using System.Text;
using FlatRedBall.Math.Geometry;
using FlatRedBall.AI.Pathfinding;
using FlatRedBall.Input;
using FlatRedBall.IO;
using FlatRedBall.Instructions;
using FlatRedBall.Math.Splines;
using FlatRedBall.Utilities;
using BitmapFont = FlatRedBall.Graphics.BitmapFont;

using Cursor = FlatRedBall.Gui.Cursor;
using GuiManager = FlatRedBall.Gui.GuiManager;

#if XNA4
using Color = Microsoft.Xna.Framework.Color;
#else
using Color = Microsoft.Xna.Framework.Graphics.Color;
#endif

#if FRB_XNA || SILVERLIGHT
using Keys = Microsoft.Xna.Framework.Input.Keys;
using Vector3 = Microsoft.Xna.Framework.Vector3;
using Texture2D = Microsoft.Xna.Framework.Graphics.Texture2D;
using Microsoft.Xna.Framework.Media;
#endif

// Generated Usings
using FlatRedBall.Broadcasting;
using RiverGame.Entities;
using FlatRedBall;
using FlatRedBall.Graphics;

namespace RiverGame.Screens
{
	public partial class OptionsScreen : Screen
	{
		// Generated Fields
		#if DEBUG
		static bool HasBeenLoadedWithGlobalContentManager = false;
		#endif
		
		private RiverGame.Entities.CheckBox UseKinectCheckBox;
		private RiverGame.Entities.CheckBox UseWiiCheckBox;
		private RiverGame.Entities.CheckBox UseMoveCheckBox;
		private Layer Layer2D;
		private RiverGame.Entities.Button BackButtonInstance;
		public event FlatRedBall.Gui.WindowEvent BackButtonInstanceClick;
		public event FlatRedBall.Gui.WindowEvent UseKinectCheckBoxClick;
		public event FlatRedBall.Gui.WindowEvent UseWiiCheckBoxClick;
		public event FlatRedBall.Gui.WindowEvent UseMoveCheckBoxClick;

		public OptionsScreen()
			: base("OptionsScreen")
		{
		}

        public override void Initialize(bool addToManagers)
        {
			// Generated Initialize
			LoadStaticContent(ContentManagerName);
			UseKinectCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			UseKinectCheckBox.Name = "UseKinectCheckBox";
			UseWiiCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			UseWiiCheckBox.Name = "UseWiiCheckBox";
			UseMoveCheckBox = new RiverGame.Entities.CheckBox(ContentManagerName, false);
			UseMoveCheckBox.Name = "UseMoveCheckBox";
			BackButtonInstance = new RiverGame.Entities.Button(ContentManagerName, false);
			BackButtonInstance.Name = "BackButtonInstance";
			BackButtonInstance.Click += OnBackButtonInstanceClick;
			BackButtonInstance.Click += OnBackButtonInstanceClickTunnel;
			UseKinectCheckBox.Click += OnUseKinectCheckBoxClick;
			UseKinectCheckBox.Click += OnUseKinectCheckBoxClickTunnel;
			UseWiiCheckBox.Click += OnUseWiiCheckBoxClick;
			UseWiiCheckBox.Click += OnUseWiiCheckBoxClickTunnel;
			UseMoveCheckBox.Click += OnUseMoveCheckBoxClick;
			UseMoveCheckBox.Click += OnUseMoveCheckBoxClickTunnel;
			
			
			PostInitialize();
			base.Initialize(addToManagers);
			if (addToManagers)
			{
				AddToManagers();
			}

        }
        
// Generated AddToManagers
		public override void AddToManagers ()
		{
			Layer2D = SpriteManager.AddLayer();
			Layer2D.UsePixelCoordinates();
			AddToManagersBottomUp();
			CustomInitialize();
		}


		public override void Activity(bool firstTimeCalled)
		{
			// Generated Activity
			if (!IsPaused)
			{
				
				UseKinectCheckBox.Activity();
				UseWiiCheckBox.Activity();
				UseMoveCheckBox.Activity();
				BackButtonInstance.Activity();
			}
			else
			{
			}
			base.Activity(firstTimeCalled);
			if (!IsActivityFinished)
			{
				CustomActivity(firstTimeCalled);
			}


				// After Custom Activity
				
            
		}

		public override void Destroy()
		{
			// Generated Destroy
			if (UseKinectCheckBox != null)
			{
				UseKinectCheckBox.Destroy();
			}
			if (UseWiiCheckBox != null)
			{
				UseWiiCheckBox.Destroy();
			}
			if (UseMoveCheckBox != null)
			{
				UseMoveCheckBox.Destroy();
			}
			if (Layer2D != null)
			{
				SpriteManager.RemoveLayer(Layer2D);
			}
			if (BackButtonInstance != null)
			{
				BackButtonInstance.Destroy();
			}
			

			base.Destroy();

			CustomDestroy();

		}

		// Generated Methods
public virtual void PostInitialize ()
{
	UseKinectCheckBox.Y = 0f;
	UseWiiCheckBox.Y = -40f;
	UseMoveCheckBox.Y = -80f;
	BackButtonInstance.DisplayText = "Back";
	BackButtonInstance.Y = -120f;
}
public virtual void AddToManagersBottomUp ()
{
	UseKinectCheckBox.AddToManagers(Layer2D);
	UseKinectCheckBox.Y = 0f;
	UseWiiCheckBox.AddToManagers(Layer2D);
	UseWiiCheckBox.Y = -40f;
	UseMoveCheckBox.AddToManagers(Layer2D);
	UseMoveCheckBox.Y = -80f;
	BackButtonInstance.AddToManagers(Layer2D);
	BackButtonInstance.DisplayText = "Back";
	BackButtonInstance.Y = -120f;
}
public virtual void ConvertToManuallyUpdated ()
{
	UseKinectCheckBox.ConvertToManuallyUpdated();
	UseWiiCheckBox.ConvertToManuallyUpdated();
	UseMoveCheckBox.ConvertToManuallyUpdated();
	BackButtonInstance.ConvertToManuallyUpdated();
}
public static void LoadStaticContent (string contentManagerName)
{
	#if DEBUG
	if (contentManagerName == FlatRedBallServices.GlobalContentManager)
	{
		HasBeenLoadedWithGlobalContentManager = true;
	}
	else if (HasBeenLoadedWithGlobalContentManager)
	{
		throw new Exception("This type has been loaded with a Global content manager, then loaded with a non-global.  This can lead to a lot of bugs");
	}
	#endif
	RiverGame.Entities.CheckBox.LoadStaticContent(contentManagerName);
	RiverGame.Entities.Button.LoadStaticContent(contentManagerName);
	CustomLoadStaticContent(contentManagerName);
}
object GetMember (string memberName)
{
	return null;
}


	}
}
