﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WiimoteLib;

namespace RiverGame.Screens
{
    public struct WiiState
    {
        public Boolean LeaningLeft, LeaningRight, LeftHandAction, RightHandAction;


    }
    public class Wii
    {
        //private Wiimote wm;
        private float wiiX;
        private float wiiY;
        private int counter;
        private WiiState state;
        public Wii() { }
        private WiimoteCollection mWC;
        private Dictionary<Guid, Wiimote> mWiimoteMap = new Dictionary<Guid, Wiimote>();
        private Wiimote LeftHand;
        private Wiimote RightHand;
        private Wiimote Board;

        private delegate void UpdateWiimoteStateDelegate(WiimoteChangedEventArgs args);
        private delegate void UpdateExtensionChangedDelegate(WiimoteExtensionChangedEventArgs args);

        public WiiState State
        {
            get { return state; }
            set { state = value; }
        }

        public Boolean Initialize()
        {

            Boolean init = false;
            counter = 0;
            wiiY = 0;
            wiiX = 0;
            state = new WiiState();
            int index = 1;
            try
            {

                //need a collection of motes since we need 2 for our case
                mWC = new WiimoteCollection();
                try
                {
                    mWC.FindAllWiimotes();
                }
                catch (WiimoteNotFoundException ex)
                {
                    //MessageBox.Show(ex.Message, "Wiimote not found error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (WiimoteException ex)
                {
                    //MessageBox.Show(ex.Message, "Wiimote error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                catch (Exception ex)
                {
                    //MessageBox.Show(ex.Message, "Unknown error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
                foreach (Wiimote wm in mWC)
                {
                    // setup the map from this wiimote's ID to that control
                    mWiimoteMap[wm.ID] = wm;
                    

                    // connect it and set it up as always
                    wm.WiimoteChanged += UpdateWiimoteChanged;
                    wm.WiimoteExtensionChanged += wm_WiimoteExtensionChanged;

                    wm.Connect();
                    wm.SetLEDs(index++);

                    Console.WriteLine("Init current WM extension type" + wm.WiimoteState.ExtensionType);

                }

                init = true;

            }
            catch (Exception e)
            {
                Console.WriteLine("Wiimote init failed");
                init = false;
            }
            return init;
        }

        void wm_WiimoteExtensionChanged(object sender, WiimoteExtensionChangedEventArgs e)
        {

            try
            {

                if (e.Inserted)
                    ((Wiimote)sender).SetReportType(InputReport.IRExtensionAccel, true);
                else
                    ((Wiimote)sender).SetReportType(InputReport.IRAccel, true);
            }
            catch
            {
                Console.WriteLine("Wii exception");
            }

        }

        void wm_WiimoteChanged(object sender, WiimoteChangedEventArgs e)
        {
          
            Wiimote wi = mWiimoteMap[((Wiimote)sender).ID];
            UpdateWiimoteChanged(sender, e);
        }

        void UpdateWiimoteChanged(object sender, WiimoteChangedEventArgs args)
        {
            WiimoteState ws = args.WiimoteState;
            Console.WriteLine("Extension:" + ws.Extension);


            switch (ws.ExtensionType)
            {

                case ExtensionType.MotionPlus:
                    Wiimote wm = ((Wiimote)sender);
                       if (ws.AccelState.Values.Y < -1.0 || ws.AccelState.Values.Y > 1.0)
                        {
                            Console.WriteLine("LeftHandAction");
                            state.LeftHandAction = true;
                            state.RightHandAction = true;

                        }
                        else
                        {
                            state.LeftHandAction = false;
                            state.RightHandAction = false;

                        }
                 
                    break;
                case ExtensionType.BalanceBoard:
                    try
                    {
                        //Board = mWiimoteMap[((Wiimote)sender).ID];
                        float Left = ws.BalanceBoardState.SensorValuesKg.BottomLeft + ws.BalanceBoardState.SensorValuesKg.TopLeft;
                        float Right = ws.BalanceBoardState.SensorValuesKg.TopRight + ws.BalanceBoardState.SensorValuesKg.BottomRight;

                        Console.WriteLine("left bottom+top:" + Left);
                        Console.WriteLine("right bottom+top:" + Right);

                        if (ws.BalanceBoardState.WeightKg>20 && ws.BalanceBoardState.CenterOfGravity.X<-5)
                        {
                            state.LeaningLeft = true;
                            state.LeaningRight = false;
                        }
                        else if (ws.BalanceBoardState.WeightKg > 20 && ws.BalanceBoardState.CenterOfGravity.X > 5)
                        {
                            state.LeaningRight = true;
                            state.LeaningLeft = false;
                        }
                        else
                        {
                            state.LeaningRight = false;
                            state.LeaningLeft = false;
                        }

                    }
                    catch
                    {
                        Console.WriteLine("Wii board failed");
                    }
                    break;

            }

        }
                
        public void Uninitialize()
        {
            foreach (Wiimote wm in mWC)
                wm.Disconnect();
        }

    }
}


